@@ -19,7 +19,6 @@ use std::{
 
 pub use config::Config;
 use datasize::DataSize;
-use itertools::Itertools;
 use prometheus::{self, Registry};
 use tracing::{debug, error, info, trace, warn};
 
@@ -31,7 +30,8 @@ use crate::{
         Component,
     },
     effect::{
-        requests::{BlockPayloadRequest, BlockProposerRequest, StateStoreRequest, StorageRequest},
+        announcements::BlockProposerAnnouncement,
+        requests::{BlockPayloadRequest, BlockProposerRequest, StorageRequest},
         EffectBuilder, EffectExt, Effects,
     },
     types::{
@@ -41,7 +41,7 @@ use crate::{
     },
     NodeRng,
 };
-use deploy_sets::BlockProposerDeploySets;
+use deploy_sets::{BlockProposerDeploySets, PruneResult};
 pub(crate) use event::{DeployInfo, Event};
 use metrics::BlockProposerMetrics;
 
@@ -103,7 +103,7 @@ impl BlockProposer {
         local_config: Config,
     ) -> Result<(Self, Effects<Event>), prometheus::Error>
     where
-        REv: From<Event> + From<StorageRequest> + From<StateStoreRequest> + Send + 'static,
+        REv: From<Event> + From<StorageRequest> + Send + 'static,
     {
         debug!(%next_finalized_block, "creating block proposer");
         let effects = effect_builder
@@ -128,7 +128,7 @@ impl BlockProposer {
 
 impl<REv> Component<REv> for BlockProposer
 where
-    REv: From<Event> + From<StorageRequest> + From<StateStoreRequest> + Send + 'static,
+    REv: From<Event> + From<StorageRequest> + From<BlockProposerAnnouncement> + Send + 'static,
 {
     type Event = Event;
     type ConstructionError = Infallible;
@@ -231,7 +231,7 @@ impl BlockProposerReady {
         event: Event,
     ) -> Effects<Event>
     where
-        REv: Send + From<StorageRequest> + From<StateStoreRequest>,
+        REv: Send + From<BlockProposerAnnouncement>,
     {
         match event {
             Event::Request(BlockProposerRequest::RequestBlockPayload(request)) => {
@@ -263,13 +263,21 @@ impl BlockProposerReady {
                 Effects::new()
             }
             Event::Prune => {
-                let pruned = self.prune(Timestamp::now());
-                debug!(%pruned, "pruned deploys from buffer");
-
                 // Re-trigger timer after `PRUNE_INTERVAL`.
-                effect_builder
+                let mut effects = effect_builder
                     .set_timeout(PRUNE_INTERVAL)
-                    .event(|_| Event::Prune)
+                    .event(|_| Event::Prune);
+
+                // Announce pruned hashes
+                let pruned_hashes = self.prune(Timestamp::now());
+                let pruned_count = pruned_hashes.total_pruned;
+                debug!(%pruned_count, "pruned deploys from buffer");
+                effects.extend(
+                    effect_builder
+                        .announce_expired_deploys(pruned_hashes.expired_hashes_to_be_announced)
+                        .ignore(),
+                );
+                effects
             }
             Event::Loaded { .. } => {
                 // This should never happen, but we can just ignore the event and carry on.
@@ -277,7 +285,19 @@ impl BlockProposerReady {
                 Effects::new()
             }
             Event::FinalizedBlock(block) => {
-                let deploys = block.deploys_and_transfers_iter().collect_vec();
+                let deploys = block
+                    .deploy_hashes()
+                    .iter()
+                    .copied()
+                    .map(DeployOrTransferHash::Deploy)
+                    .chain(
+                        block
+                            .transfer_hashes()
+                            .iter()
+                            .copied()
+                            .map(DeployOrTransferHash::Transfer),
+                    )
+                    .collect();
                 let mut height = block.height();
 
                 if height > self.sets.next_finalized {
@@ -492,8 +512,9 @@ impl BlockProposerReady {
         Arc::new(appendable_block.into_block_payload(accusations, random_bit))
     }
 
-    /// Prunes expired deploy information from the BlockProposer, returns the total deploys pruned.
-    fn prune(&mut self, current_instant: Timestamp) -> usize {
+    /// Prunes expired deploy information from the BlockProposer, returns the hashes of deploys
+    /// pruned.
+    fn prune(&mut self, current_instant: Timestamp) -> PruneResult {
         self.sets.prune(current_instant)
     }
 
