@@ -1,8 +1,9 @@
-use casper_execution_engine::{
-    core::engine_state::executable_deploy_item::ExecutableDeployItem, shared::gas::Gas,
-};
+use std::time::Duration;
+
+use casper_execution_engine::core::engine_state::executable_deploy_item::ExecutableDeployItem;
 use casper_types::{
-    bytesrepr::Bytes, runtime_args, system::standard_payment::ARG_AMOUNT, RuntimeArgs, SecretKey,
+    bytesrepr::Bytes, runtime_args, system::standard_payment::ARG_AMOUNT, Gas, RuntimeArgs,
+    SecretKey,
 };
 use itertools::Itertools;
 
@@ -51,6 +52,7 @@ fn generate_transfer(
         payment,
         session,
         &secret_key,
+        None,
     )
 }
 
@@ -85,6 +87,7 @@ fn generate_deploy(
         payment,
         session,
         &secret_key,
+        None,
     )
 }
 
@@ -103,12 +106,6 @@ impl From<StorageRequest> for Event {
     }
 }
 
-impl From<StateStoreRequest> for Event {
-    fn from(_: StateStoreRequest) -> Self {
-        unreachable!("no state store requests in block proposer unit tests")
-    }
-}
-
 #[test]
 fn should_add_and_take_deploys() {
     let creation_time = Timestamp::from(100);
@@ -329,15 +326,26 @@ fn should_successfully_prune() {
 
     // test for retained values
     let pruned = proposer.prune(test_time);
-    assert_eq!(pruned, 0);
+    assert_eq!(pruned.total_pruned, 0);
 
     assert_eq!(proposer.sets.pending_deploys.len(), 3);
     assert_eq!(proposer.sets.finalized_deploys.len(), 1);
     assert!(proposer.sets.finalized_deploys.contains_key(deploy1.id()));
 
     // now move the clock to make some things expire
-    let pruned = proposer.prune(expired_time);
-    assert_eq!(pruned, 3);
+    let mut pruned = proposer.prune(expired_time);
+    assert_eq!(pruned.total_pruned, 3); // Three deploys pruned
+
+    // Expiration announcements created for two deploys only: "deploy2" and "deploy3"
+    // because "deploy4" did not expire and "deploy1" has been finalized
+    assert_eq!(pruned.expired_hashes_to_be_announced.len(), 2);
+    let mut hashes_to_be_announced = vec![*deploy2.id(), *deploy3.id()];
+    hashes_to_be_announced.sort();
+    pruned.expired_hashes_to_be_announced.sort();
+    assert_eq!(
+        pruned.expired_hashes_to_be_announced,
+        hashes_to_be_announced
+    );
 
     assert_eq!(proposer.sets.pending_deploys.len(), 1); // deploy4 is still valid
     assert_eq!(proposer.sets.finalized_deploys.len(), 0);
