@@ -26,21 +26,18 @@ use serde::{Deserialize, Serialize};
 use tokio::task;
 use tracing::{debug, error, info, trace, warn};
 
-use casper_execution_engine::{
-    core::engine_state::{
-        self,
-        genesis::GenesisResult,
-        upgrade::{UpgradeConfig, UpgradeResult},
-    },
-    shared::stored_value::StoredValue,
+use casper_execution_engine::core::engine_state::{
+    self,
+    genesis::GenesisSuccess,
+    upgrade::{UpgradeConfig, UpgradeSuccess},
 };
-use casper_types::{bytesrepr::FromBytes, EraId, ProtocolVersion};
+use casper_hashing::Digest;
+use casper_types::{bytesrepr::FromBytes, EraId, ProtocolVersion, StoredValue};
 
 #[cfg(test)]
 use crate::utils::RESOURCES_PATH;
 use crate::{
-    components::Component,
-    crypto::hash::Digest,
+    components::{contract_runtime::ExecutionPreState, Component},
     effect::{
         announcements::ChainspecLoaderAnnouncement,
         requests::{
@@ -51,7 +48,7 @@ use crate::{
     reactor::ReactorExit,
     types::{
         chainspec::{Error, ProtocolConfig, CHAINSPEC_NAME},
-        ActivationPoint, Block, BlockHeader, Chainspec, ChainspecInfo, ExitCode,
+        ActivationPoint, Block, BlockHash, BlockHeader, Chainspec, ChainspecInfo, ExitCode,
     },
     utils::{self, Loadable},
     NodeRng,
@@ -61,23 +58,21 @@ const UPGRADE_CHECK_INTERVAL: Duration = Duration::from_secs(60);
 
 /// `ChainspecHandler` events.
 #[derive(Debug, From, Serialize)]
-pub enum Event {
+pub(crate) enum Event {
     /// The result of getting the highest block from storage.
     Initialize {
         maybe_highest_block: Option<Box<Block>>,
     },
     /// The result of contract runtime running the genesis process.
-    CommitGenesisResult(#[serde(skip_serializing)] Result<GenesisResult, engine_state::Error>),
+    CommitGenesisResult(#[serde(skip_serializing)] Result<GenesisSuccess, engine_state::Error>),
     /// The result of contract runtime running the upgrade process.
-    UpgradeResult(#[serde(skip_serializing)] Result<UpgradeResult, engine_state::Error>),
+    UpgradeResult(#[serde(skip_serializing)] Result<UpgradeSuccess, engine_state::Error>),
     #[from]
     Request(ChainspecLoaderRequest),
     /// Check config dir to see if an upgrade activation point is available, and if so announce it.
     CheckForNextUpgrade,
     /// If the result of checking for an upgrade is successful, it is passed here.
     GotNextUpgrade(NextUpgrade),
-    /// The result of the `ChainspecHandler` putting a `Chainspec` to the storage component.
-    PutToStorage { version: ProtocolVersion },
 }
 
 impl Display for Event {
@@ -103,9 +98,6 @@ impl Display for Event {
             Event::GotNextUpgrade(next_upgrade) => {
                 write!(formatter, "got {}", next_upgrade)
             }
-            Event::PutToStorage { version } => {
-                write!(formatter, "put chainspec {} to storage", version)
-            }
         }
     }
 }
@@ -157,15 +149,15 @@ impl Display for NextUpgrade {
 
 /// Basic information about the current run of the node software.
 #[derive(Clone, Debug)]
-pub struct CurrentRunInfo {
-    pub activation_point: ActivationPoint,
-    pub protocol_version: ProtocolVersion,
-    pub initial_state_root_hash: Digest,
-    pub last_emergency_restart: Option<EraId>,
+pub(crate) struct CurrentRunInfo {
+    pub(crate) activation_point: ActivationPoint,
+    pub(crate) protocol_version: ProtocolVersion,
+    pub(crate) initial_state_root_hash: Digest,
+    pub(crate) last_emergency_restart: Option<EraId>,
 }
 
 #[derive(Clone, DataSize, Debug)]
-pub struct ChainspecLoader {
+pub(crate) struct ChainspecLoader {
     chainspec: Arc<Chainspec>,
     /// The path to the folder where all chainspec and upgrade_point files will be stored in
     /// subdirs corresponding to their versions.
@@ -305,7 +297,7 @@ impl ChainspecLoader {
     }
 
     /// Returns whether the current node instance is started immediately after an upgrade –
-    /// i.e. whether the last/highest block stored is a block that triggerred the upgrade.
+    /// i.e. whether the last/highest block stored is a block that triggered the upgrade.
     pub(crate) fn after_upgrade(&self) -> bool {
         self.after_upgrade
     }
@@ -317,6 +309,36 @@ impl ChainspecLoader {
         self.initial_state_root_hash
     }
 
+    /// The state the first block executed after startup will use. It contains:
+    ///   1. The state root to use when executing the next batch of deploys. See
+    /// [`Self::initial_state_root_hash`].
+    ///   2. The height of the next block to be added to the chain.
+    ///     - At genesis the height is 0.
+    ///     - Otherwise it is the successor of the last upgrade block.
+    ///   3. The parent hash for the first block to use when executing.
+    ///     - At genesis this is `[0u8; 32]`
+    ///     - Otherwise it is the block hash of the highest block.
+    ///   4. The _accumulated seed_ to be used in pseudo-random number generation and entropy will
+    /// be added.
+    ///     - At genesis this is `[0u8; 32]`
+    ///     - Otherwise it is the accumulated seed of the highest block.
+    pub(crate) fn initial_execution_pre_state(&self) -> ExecutionPreState {
+        match self.initial_block() {
+            None => ExecutionPreState::new(
+                0,
+                self.initial_state_root_hash(),
+                BlockHash::new(Digest::from([0u8; Digest::LENGTH])),
+                Digest::from([0u8; Digest::LENGTH]),
+            ),
+            Some(block) => ExecutionPreState::new(
+                block.height() + 1,
+                self.initial_state_root_hash(),
+                *block.hash(),
+                block.header().accumulated_seed(),
+            ),
+        }
+    }
+
     pub(crate) fn chainspec(&self) -> &Arc<Chainspec> {
         &self.chainspec
     }
@@ -434,21 +456,36 @@ impl ChainspecLoader {
             return Effects::new();
         }
 
-        let unplanned_shutdown = match self.next_upgrade {
-            Some(ref next_upgrade) => highest_block_era_id < next_upgrade.activation_point.era_id(),
-            None => true,
+        let (unplanned_shutdown, should_upgrade_if_valid_run) = match self.next_upgrade {
+            Some(ref next_upgrade) => {
+                let unplanned_shutdown =
+                    highest_block_era_id < next_upgrade.activation_point.era_id();
+                let should_upgrade_if_valid_run = highest_block.header().is_switch_block()
+                    && highest_block_era_id.successor() == next_upgrade.activation_point.era_id();
+                (unplanned_shutdown, should_upgrade_if_valid_run)
+            }
+            None => (true, false),
         };
 
         if unplanned_shutdown {
             if previous_protocol_version == self.chainspec.protocol_config.version {
                 // This is a valid run, restarted after an unplanned shutdown.
-                self.initial_state_root_hash = *highest_block.state_root_hash();
-                info!(
-                    %current_chainspec_activation_point,
-                    %highest_block,
-                    "valid run after an unplanned shutdown before upgrade due"
-                );
-                self.reactor_exit = Some(ReactorExit::ProcessShouldContinue);
+                if should_upgrade_if_valid_run {
+                    warn!(
+                        %current_chainspec_activation_point,
+                        %highest_block,
+                        "valid run after an unplanned shutdown when upgrade was due"
+                    );
+                    self.reactor_exit = Some(ReactorExit::ProcessShouldExit(ExitCode::Success));
+                } else {
+                    self.initial_state_root_hash = *highest_block.state_root_hash();
+                    info!(
+                        %current_chainspec_activation_point,
+                        %highest_block,
+                        "valid run after an unplanned shutdown before upgrade due"
+                    );
+                    self.reactor_exit = Some(ReactorExit::ProcessShouldContinue);
+                }
             } else {
                 // This is an invalid run, where we previously forked and missed an upgrade.
                 warn!(
@@ -507,11 +544,9 @@ impl ChainspecLoader {
             })
             .unwrap_or_default();
         Box::new(UpgradeConfig::new(
-            (*block.state_root_hash()).into(),
+            *block.state_root_hash(),
             previous_version,
             new_version,
-            Some(self.chainspec.wasm_config),
-            Some(self.chainspec.system_costs_config),
             Some(self.chainspec.protocol_config.activation_point.era_id()),
             Some(self.chainspec.core_config.validator_slots),
             Some(self.chainspec.core_config.auction_delay),
@@ -524,66 +559,43 @@ impl ChainspecLoader {
 
     fn handle_commit_genesis_result(
         &mut self,
-        result: Result<GenesisResult, engine_state::Error>,
-    ) -> Effects<Event> {
+        result: Result<GenesisSuccess, engine_state::Error>,
+    ) {
         match result {
-            Ok(genesis_result) => match genesis_result {
-                GenesisResult::RootNotFound
-                | GenesisResult::KeyNotFound(_)
-                | GenesisResult::TypeMismatch(_)
-                | GenesisResult::Serialization(_) => {
-                    error!("failed to commit genesis: {}", genesis_result);
-                    self.reactor_exit = Some(ReactorExit::ProcessShouldExit(ExitCode::Abort));
-                }
-                GenesisResult::Success {
-                    post_state_hash,
-                    effect,
-                } => {
-                    info!("chainspec name {}", self.chainspec.network_config.name);
-                    info!("genesis state root hash {}", post_state_hash);
-                    trace!(%post_state_hash, ?effect);
-                    self.reactor_exit = Some(ReactorExit::ProcessShouldContinue);
-                    self.initial_state_root_hash = post_state_hash.into();
-                }
-            },
+            Ok(GenesisSuccess {
+                post_state_hash,
+                execution_effect,
+            }) => {
+                info!("chainspec name {}", self.chainspec.network_config.name);
+                info!("genesis state root hash {}", post_state_hash);
+                trace!(%post_state_hash, ?execution_effect);
+                self.reactor_exit = Some(ReactorExit::ProcessShouldContinue);
+                self.initial_state_root_hash = post_state_hash;
+            }
             Err(error) => {
                 error!("failed to commit genesis: {}", error);
                 self.reactor_exit = Some(ReactorExit::ProcessShouldExit(ExitCode::Abort));
             }
         }
-        Effects::new()
     }
 
-    fn handle_upgrade_result(
-        &mut self,
-        result: Result<UpgradeResult, engine_state::Error>,
-    ) -> Effects<Event> {
+    fn handle_upgrade_result(&mut self, result: Result<UpgradeSuccess, engine_state::Error>) {
         match result {
-            Ok(upgrade_result) => match upgrade_result {
-                UpgradeResult::RootNotFound
-                | UpgradeResult::KeyNotFound(_)
-                | UpgradeResult::TypeMismatch(_)
-                | UpgradeResult::Serialization(_) => {
-                    error!("failed to upgrade contract runtime: {}", upgrade_result);
-                    self.reactor_exit = Some(ReactorExit::ProcessShouldExit(ExitCode::Abort));
-                }
-                UpgradeResult::Success {
-                    post_state_hash,
-                    effect,
-                } => {
-                    info!("chainspec name {}", self.chainspec.network_config.name);
-                    info!("state root hash {}", post_state_hash);
-                    trace!(%post_state_hash, ?effect);
-                    self.reactor_exit = Some(ReactorExit::ProcessShouldContinue);
-                    self.initial_state_root_hash = post_state_hash.into();
-                }
-            },
+            Ok(UpgradeSuccess {
+                post_state_hash,
+                execution_effect,
+            }) => {
+                info!("chainspec name {}", self.chainspec.network_config.name);
+                info!("state root hash {}", post_state_hash);
+                trace!(%post_state_hash, ?execution_effect);
+                self.reactor_exit = Some(ReactorExit::ProcessShouldContinue);
+                self.initial_state_root_hash = post_state_hash;
+            }
             Err(error) => {
                 error!("failed to upgrade contract runtime: {}", error);
                 self.reactor_exit = Some(ReactorExit::ProcessShouldExit(ExitCode::Abort));
             }
         }
-        Effects::new()
     }
 
     fn new_chainspec_info(&self) -> ChainspecInfo {
@@ -673,8 +685,14 @@ where
             Event::Initialize {
                 maybe_highest_block: highest_block,
             } => self.handle_initialize(effect_builder, highest_block),
-            Event::CommitGenesisResult(result) => self.handle_commit_genesis_result(result),
-            Event::UpgradeResult(result) => self.handle_upgrade_result(result),
+            Event::CommitGenesisResult(result) => {
+                self.handle_commit_genesis_result(result);
+                Effects::new()
+            }
+            Event::UpgradeResult(result) => {
+                self.handle_upgrade_result(result);
+                Effects::new()
+            }
             Event::Request(ChainspecLoaderRequest::GetChainspecInfo(responder)) => {
                 responder.respond(self.new_chainspec_info()).ignore()
             }
@@ -683,12 +701,6 @@ where
             }
             Event::CheckForNextUpgrade => self.check_for_next_upgrade(effect_builder),
             Event::GotNextUpgrade(next_upgrade) => self.handle_got_next_upgrade(next_upgrade),
-            Event::PutToStorage { version } => {
-                debug!("stored chainspec {}", version);
-                effect_builder
-                    .commit_genesis(Arc::clone(&self.chainspec))
-                    .event(Event::CommitGenesisResult)
-            }
         }
     }
 }
@@ -816,9 +828,7 @@ mod tests {
     use super::*;
     use crate::{
         logging,
-        reactor::{
-            participating::Event as ParticipatingEvent, EventQueueHandle, QueueKind, Scheduler,
-        },
+        reactor::{participating::ParticipatingEvent, EventQueueHandle, QueueKind, Scheduler},
         testing::TestRng,
         types::chainspec::CHAINSPEC_NAME,
     };
@@ -1006,12 +1016,14 @@ mod tests {
         assert!(maybe_next_point(&current).is_none());
     }
 
+    #[cfg(test)]
     struct TestFixture {
         chainspec_loader: ChainspecLoader,
         effect_builder: EffectBuilder<ParticipatingEvent>,
     }
 
     impl TestFixture {
+        #[cfg(test)]
         fn new() -> Self {
             let _ = logging::init();
 
@@ -1041,6 +1053,7 @@ mod tests {
         }
 
         /// Returns the current chainspec's activation point.
+        #[cfg(test)]
         fn current_activation_point(&self) -> EraId {
             self.chainspec_loader
                 .chainspec
@@ -1050,11 +1063,13 @@ mod tests {
         }
 
         /// Returns the current chainspec's protocol version.
+        #[cfg(test)]
         fn current_protocol_version(&self) -> ProtocolVersion {
             self.chainspec_loader.chainspec.protocol_config.version
         }
 
         /// Returns a protocol version earlier than the current chainspec's version.
+        #[cfg(test)]
         fn earlier_protocol_version(&self) -> ProtocolVersion {
             ProtocolVersion::from_parts(
                 self.current_protocol_version().value().major,
@@ -1064,6 +1079,7 @@ mod tests {
         }
 
         /// Returns a protocol version later than the current chainspec's version.
+        #[cfg(test)]
         fn later_protocol_version(&self) -> ProtocolVersion {
             ProtocolVersion::from_parts(
                 self.current_protocol_version().value().major,
@@ -1073,6 +1089,7 @@ mod tests {
         }
 
         /// Sets a valid value for the next upgrade in the chainspec loader.
+        #[cfg(test)]
         fn set_next_upgrade(&mut self, era_diff: u64) {
             self.chainspec_loader.next_upgrade = Some(NextUpgrade {
                 activation_point: ActivationPoint::EraId(
@@ -1084,6 +1101,7 @@ mod tests {
 
         /// Calls `handle_initialize()` on the chainspec loader, asserting the provided block has
         /// been recorded and that the expected number of effects were returned.
+        #[cfg(test)]
         fn assert_handle_initialize(
             &mut self,
             maybe_highest_block: Option<Block>,
@@ -1100,12 +1118,14 @@ mod tests {
 
         /// Asserts that the chainspec loader indicates initialization is ongoing, i.e. that
         /// `chainspec_loader.reactor_exit` is `None`.
+        #[cfg(test)]
         fn assert_initialization_incomplete(&self) {
             assert!(self.chainspec_loader.reactor_exit.is_none())
         }
 
         /// Asserts that the chainspec loader indicates initialization is complete and the node
         /// process should not stop.
+        #[cfg(test)]
         fn assert_process_should_continue(&self) {
             assert_eq!(
                 self.chainspec_loader.reactor_exit,
@@ -1114,6 +1134,7 @@ mod tests {
         }
 
         /// Asserts that the chainspec loader indicates the process should stop to downgrade.
+        #[cfg(test)]
         fn assert_process_should_downgrade(&self) {
             assert_eq!(
                 self.chainspec_loader.reactor_exit,
@@ -1122,6 +1143,7 @@ mod tests {
         }
 
         /// Asserts that the chainspec loader indicates the process should stop to upgrade.
+        #[cfg(test)]
         fn assert_process_should_upgrade(&self) {
             assert_eq!(
                 self.chainspec_loader.reactor_exit,
@@ -1130,6 +1152,7 @@ mod tests {
         }
 
         /// Asserts that the chainspec loader indicates the process should stop with an error.
+        #[cfg(test)]
         fn assert_process_should_abort(&self) {
             assert_eq!(
                 self.chainspec_loader.reactor_exit,
