@@ -18,10 +18,6 @@ use std::{
 };
 
 use anyhow::Error;
-use blake2::{
-    digest::{Update, VariableOutput},
-    VarBlake2b,
-};
 use datasize::DataSize;
 use futures::FutureExt;
 use itertools::Itertools;
@@ -29,8 +25,10 @@ use prometheus::Registry;
 use rand::Rng;
 use tracing::{debug, error, info, trace, warn};
 
+use casper_hashing::Digest;
 use casper_types::{AsymmetricType, EraId, PublicKey, SecretKey, U512};
 
+pub use self::era::Era;
 use crate::{
     components::consensus::{
         cl_context::{ClContext, Keypair},
@@ -41,25 +39,24 @@ use crate::{
         },
         metrics::ConsensusMetrics,
         traits::NodeIdT,
+        validator_change::ValidatorChanges,
         ActionId, Config, ConsensusMessage, Event, NewBlockPayload, ReactorEventT, ResolveValidity,
-        TimerId,
+        TimerId, ValidatorChange,
     },
-    crypto::hash::Digest,
     effect::{
-        requests::{BlockValidationRequest, StorageRequest},
+        announcements::ControlAnnouncement,
+        requests::{BlockValidationRequest, ContractRuntimeRequest, StorageRequest},
         EffectBuilder, EffectExt, Effects, Responder,
     },
     fatal,
     types::{
-        ActivationPoint, BlockHash, BlockHeader, DeployHash, DeployOrTransferHash,
+        ActivationPoint, BlockHash, BlockHeader, Deploy, DeployHash, DeployOrTransferHash,
         FinalitySignature, FinalizedBlock, TimeDiff, Timestamp,
     },
     utils::WithDir,
     NodeRng,
 };
 
-pub use self::era::Era;
-
 /// The delay in milliseconds before we shutdown after the number of faulty validators exceeded the
 /// fault tolerance threshold.
 const FTT_EXCEEDED_SHUTDOWN_DELAY_MILLIS: u64 = 60 * 1000;
@@ -236,17 +233,21 @@ where
         }
     }
 
+    /// Returns a list of status changes of active validators.
+    pub(super) fn get_validator_changes(
+        &self,
+    ) -> BTreeMap<PublicKey, Vec<(EraId, ValidatorChange)>> {
+        let mut result: BTreeMap<PublicKey, Vec<(EraId, ValidatorChange)>> = BTreeMap::new();
+        for ((_, era0), (era_id, era1)) in self.active_eras.iter().tuple_windows() {
+            for (pub_key, change) in ValidatorChanges::new(era0, era1).0 {
+                result.entry(pub_key).or_default().push((*era_id, change));
+            }
+        }
+        result
+    }
+
     fn era_seed(booking_block_hash: BlockHash, key_block_seed: Digest) -> u64 {
-        let mut result = [0; Digest::LENGTH];
-        let mut hasher = VarBlake2b::new(Digest::LENGTH).expect("should create hasher");
-
-        hasher.update(booking_block_hash);
-        hasher.update(key_block_seed);
-
-        hasher.finalize_variable(|slice| {
-            result.copy_from_slice(slice);
-        });
-
+        let result = Digest::hash_pair(booking_block_hash, key_block_seed).value();
         u64::from_le_bytes(result[0..std::mem::size_of::<u64>()].try_into().unwrap())
     }
 
@@ -363,6 +364,7 @@ where
             start_height,
             new_faulty,
             faulty,
+            inactive,
             validators,
         );
 
@@ -459,6 +461,7 @@ where
         activation_era_validators: BTreeMap<PublicKey, U512>,
     ) -> HashMap<EraId, ProtocolOutcomes<I, ClContext>> {
         let mut result_map = HashMap::new();
+        let now = Timestamp::now();
 
         for era_id in self.iter_past(self.current_era, self.bonded_eras().saturating_mul(2)) {
             let new_faulty;
@@ -517,7 +520,7 @@ where
 
             let results = self.new_era(
                 era_id,
-                Timestamp::now(),
+                now,
                 validators,
                 new_faulty,
                 faulty,
@@ -536,7 +539,7 @@ where
         }
         let active_era_outcomes = self.active_eras[&self.current_era]
             .consensus
-            .handle_is_current();
+            .handle_is_current(now);
         result_map
             .entry(self.current_era)
             .or_default()
@@ -588,6 +591,11 @@ where
             .consensus
             .validators_with_evidence()
     }
+
+    /// Returns this node's validator key.
+    pub(crate) fn public_key(&self) -> &PublicKey {
+        &self.public_signing_key
+    }
 }
 
 /// Returns an era ID in which the booking block for `era_id` lives, if we can use it.
@@ -798,21 +806,37 @@ where
             trace!(era = era_id.value(), "executed block in old era");
             return effects;
         }
-        if block_header.is_switch_block() && !self.should_upgrade_after(&era_id) {
-            // if the block is a switch block, we have to get the validators for the new era and
-            // create it, before we can say we handled the block
-            let new_era_id = era_id.successor();
-            let effect = get_booking_block_hash(
-                self.effect_builder,
-                new_era_id,
-                self.era_supervisor.protocol_config.auction_delay,
-                self.era_supervisor.protocol_config.last_activation_point,
-            )
-            .event(move |booking_block_hash| Event::CreateNewEra {
-                switch_block_header: Box::new(block_header),
-                booking_block_hash: Ok(booking_block_hash),
-            });
-            effects.extend(effect);
+        if block_header.is_switch_block() {
+            if let Some(era) = self.era_supervisor.active_eras.get_mut(&era_id) {
+                // This was the era's last block. Schedule deactivating this era.
+                let delay = Timestamp::now()
+                    .saturating_diff(block_header.timestamp())
+                    .into();
+                let faulty_num = era.consensus.validators_with_evidence().len();
+                let deactivate_era = move |_| Event::DeactivateEra {
+                    era_id,
+                    faulty_num,
+                    delay,
+                };
+                effects.extend(self.effect_builder.set_timeout(delay).event(deactivate_era));
+            } else {
+                error!(era = era_id.value(), %block_header, "executed block in uninitialized era");
+            }
+            // If it's not the last block before an upgrade, initialize the next era.
+            if !self.should_upgrade_after(&era_id) {
+                let new_era_id = era_id.successor();
+                let effect = get_booking_block_hash(
+                    self.effect_builder,
+                    new_era_id,
+                    self.era_supervisor.protocol_config.auction_delay,
+                    self.era_supervisor.protocol_config.last_activation_point,
+                )
+                .event(move |booking_block_hash| Event::CreateNewEra {
+                    switch_block_header: Box::new(block_header),
+                    booking_block_hash: Ok(booking_block_hash),
+                });
+                effects.extend(effect);
+            }
         }
         effects
     }
@@ -907,10 +931,11 @@ where
             .chain(&new_faulty)
             .cloned()
             .collect();
+        let now = Timestamp::now(); // TODO: This should be passed in.
         #[allow(clippy::integer_arithmetic)] // Block height should never reach u64::MAX.
         let mut outcomes = self.era_supervisor.new_era(
             era_id,
-            Timestamp::now(), // TODO: This should be passed in.
+            now,
             next_era_validators_weights.clone(),
             new_faulty,
             faulty,
@@ -922,7 +947,7 @@ where
         outcomes.extend(
             self.era_supervisor.active_eras[&era_id]
                 .consensus
-                .handle_is_current(),
+                .handle_is_current(now),
         );
         self.handle_consensus_outcomes(era_id, outcomes)
     }
@@ -941,7 +966,7 @@ where
         let mut effects = Effects::new();
         if !valid {
             warn!(
-                %sender,
+                peer_id=%sender,
                 era = %era_id.value(),
                 "invalid consensus value; disconnecting from the sender"
             );
@@ -1023,6 +1048,17 @@ where
                     .send_message(to, message.into())
                     .ignore()
             }
+            ProtocolOutcome::CreatedMessageToRandomPeer(payload) => {
+                let message = ConsensusMessage::Protocol { era_id, payload };
+                let effect_builder = self.effect_builder;
+                async move {
+                    let peers = effect_builder.get_peers_in_random_order().await;
+                    if let Some(to) = peers.into_iter().next() {
+                        effect_builder.send_message(to, message.into()).await;
+                    }
+                }
+                .ignore()
+            }
             ProtocolOutcome::ScheduleTimer(timestamp, timer_id) => {
                 let timediff = timestamp.saturating_diff(Timestamp::now());
                 self.effect_builder
@@ -1080,15 +1116,8 @@ where
                 // the block or seen as equivocating via the consensus protocol gets faulty.
                 let era_end = terminal_block_data.map(|tbd| EraReport {
                     rewards: tbd.rewards,
-                    // TODO: This is a temporary change to disable slashing; we just report all
-                    // equivocators as "inactive" instead. Change this back and put
-                    // era.accusations() into equivocators instead of inactive_validators.
-                    equivocators: vec![],
-                    inactive_validators: tbd
-                        .inactive_validators
-                        .into_iter()
-                        .chain(era.accusations())
-                        .collect(),
+                    equivocators: era.accusations(),
+                    inactive_validators: tbd.inactive_validators,
                 });
                 let finalized_block = FinalizedBlock::new(
                     Arc::try_unwrap(value).unwrap_or_else(|arc| (*arc).clone()),
@@ -1098,7 +1127,9 @@ where
                     era.start_height + relative_height,
                     proposer,
                 );
-                info!(?finalized_block, "finalized block");
+                info!(era_id=?finalized_block.era_id(),
+                        height=?finalized_block.height(),
+                        timestamp=?finalized_block.timestamp(), "finalized block");
                 self.era_supervisor
                     .metrics
                     .finalized_block(&finalized_block);
@@ -1111,19 +1142,9 @@ where
                     .era_supervisor
                     .next_block_height
                     .max(finalized_block.height() + 1);
-                if finalized_block.era_report().is_some() {
-                    // This was the era's last block. Schedule deactivating this era.
-                    let delay = Timestamp::now().saturating_diff(timestamp).into();
-                    let faulty_num = era.consensus.validators_with_evidence().len();
-                    let deactivate_era = move |_| Event::DeactivateEra {
-                        era_id,
-                        faulty_num,
-                        delay,
-                    };
-                    effects.extend(self.effect_builder.set_timeout(delay).event(deactivate_era));
-                }
                 // Request execution of the finalized block.
-                effects.extend(self.effect_builder.execute_block(finalized_block).ignore());
+                let effect_builder = self.effect_builder;
+                effects.extend(execute_finalized_block(effect_builder, finalized_block).ignore());
                 self.era_supervisor.update_consensus_pause();
                 effects
             }
@@ -1270,18 +1291,76 @@ where
     }
 }
 
+async fn get_deploys_or_transfers<REv>(
+    effect_builder: EffectBuilder<REv>,
+    hashes: Vec<DeployHash>,
+) -> Option<Vec<Deploy>>
+where
+    REv: From<StorageRequest>,
+{
+    let mut deploys_or_transfer: Vec<Deploy> = Vec::with_capacity(hashes.len());
+    for maybe_deploy_or_transfer in effect_builder.get_deploys_from_storage(hashes).await {
+        if let Some(deploy_or_transfer) = maybe_deploy_or_transfer {
+            deploys_or_transfer.push(deploy_or_transfer)
+        } else {
+            return None;
+        }
+    }
+    Some(deploys_or_transfer)
+}
+
+async fn execute_finalized_block<REv>(
+    effect_builder: EffectBuilder<REv>,
+    finalized_block: FinalizedBlock,
+) where
+    REv: From<StorageRequest> + From<ControlAnnouncement> + From<ContractRuntimeRequest>,
+{
+    // Get all deploys in order they appear in the finalized block.
+    let deploys =
+        match get_deploys_or_transfers(effect_builder, finalized_block.deploy_hashes().to_owned())
+            .await
+        {
+            Some(deploys) => deploys,
+            None => {
+                fatal!(
+                    effect_builder,
+                    "Could not fetch deploys for finalized block: {:?}",
+                    finalized_block
+                )
+                .await;
+                return;
+            }
+        };
+
+    // Get all transfers in order they appear in the finalized block.
+    let transfers = match get_deploys_or_transfers(
+        effect_builder,
+        finalized_block.transfer_hashes().to_owned(),
+    )
+    .await
+    {
+        Some(transfers) => transfers,
+        None => {
+            fatal!(
+                effect_builder,
+                "Could not fetch transfers for finalized block: {:?}",
+                finalized_block
+            )
+            .await;
+            return;
+        }
+    };
+
+    effect_builder
+        .enqueue_block_for_execution(finalized_block, deploys, transfers)
+        .await
+}
+
 /// Computes the instance ID for an era, given the era ID and the chainspec hash.
 fn instance_id(protocol_config: &ProtocolConfig, era_id: EraId) -> Digest {
-    let mut result = [0; Digest::LENGTH];
-    let mut hasher = VarBlake2b::new(Digest::LENGTH).expect("should create hasher");
-
-    hasher.update(protocol_config.chainspec_hash.as_ref());
-    hasher.update(era_id.to_le_bytes());
-
-    hasher.finalize_variable(|slice| {
-        result.copy_from_slice(slice);
-    });
-    result.into()
+    Digest::hash_pair(protocol_config.chainspec_hash, era_id.to_le_bytes())
+        .value()
+        .into()
 }
 
 /// The number of past eras whose validators are still bonded. After this many eras, a former
