@@ -28,6 +28,8 @@ use crate::{
 use super::{
     endorsement::{Endorsement, EndorsementError},
     evidence::Evidence,
+    state::Observation,
+    validators::ValidatorIndex,
 };
 
 /// If a lot of rounds were skipped between two blocks, log at most this many.
@@ -219,7 +221,7 @@ impl<C: Context> Highway<C> {
             self.instance_id,
         );
         self.active_validator = Some(av);
-        effects
+        self.add_new_own_vertices(effects, current_time)
     }
 
     /// Turns this instance into a passive observer, that does not create any new vertices.
@@ -372,6 +374,37 @@ impl<C: Context> Highway<C> {
         }
     }
 
+    /// Returns a vertex by a validator with the requested sequence number.
+    pub(crate) fn get_dependency_by_index(
+        &self,
+        vid: ValidatorIndex,
+        unit_seq: u64,
+    ) -> GetDepOutcome<C> {
+        let obs = match self.state.panorama().get(vid) {
+            Some(obs) => obs,
+            None => return GetDepOutcome::None,
+        };
+        match obs {
+            Observation::None => GetDepOutcome::None,
+            Observation::Faulty => match self.state.maybe_fault(vid) {
+                None | Some(Fault::Banned) => GetDepOutcome::None,
+                Some(Fault::Direct(ev)) => {
+                    GetDepOutcome::Vertex(ValidVertex(Vertex::Evidence(ev.clone())))
+                }
+                Some(Fault::Indirect) => match self.validators.id(vid) {
+                    Some(vid) => GetDepOutcome::Evidence(vid.clone()),
+                    None => GetDepOutcome::None,
+                },
+            },
+            Observation::Correct(last_seen) => self
+                .state
+                .find_ancestor_unit(last_seen, unit_seq)
+                .and_then(|req_hash| self.state.wire_unit(req_hash, self.instance_id))
+                .map(|swunit| GetDepOutcome::Vertex(ValidVertex(Vertex::Unit(swunit))))
+                .unwrap_or_else(|| GetDepOutcome::None),
+        }
+    }
+
     pub(crate) fn handle_timer(&mut self, timestamp: Timestamp) -> Vec<Effect<C>> {
         let instance_id = self.instance_id;
 
@@ -500,7 +533,19 @@ impl<C: Context> Highway<C> {
         F: FnOnce(&mut ActiveValidator<C>, &State<C>) -> Vec<Effect<C>>,
     {
         let effects = f(self.active_validator.as_mut()?, &self.state);
-        let mut result = vec![];
+        Some(self.add_new_own_vertices(effects, timestamp))
+    }
+
+    /// Handles all `NewVertex` effects and adds the vertices to the protocol state.
+    ///
+    /// This needs to be applied to all effects created by `ActiveValidator`, so that new vertices
+    /// are not interpreted as coming from a doppelgänger.
+    fn add_new_own_vertices(
+        &mut self,
+        effects: Vec<Effect<C>>,
+        timestamp: Timestamp,
+    ) -> Vec<Effect<C>> {
+        let mut result = Vec::with_capacity(effects.len());
         for effect in &effects {
             match effect {
                 Effect::NewVertex(vv) => {
@@ -511,7 +556,7 @@ impl<C: Context> Highway<C> {
             }
         }
         result.extend(effects);
-        Some(result)
+        result
     }
 
     /// Performs initial validation and returns an error if `vertex` is invalid. (See
@@ -985,4 +1030,27 @@ pub(crate) mod tests {
         // Verify that sending a Ping from a non-existing validator does not panic.
         assert!(!highway.has_vertex(&ping));
     }
+
+    #[test]
+    fn own_initial_ping_is_not_from_doppelganger() {
+        let now: Timestamp = 500.into();
+        let later = 501.into();
+
+        let state: State<TestContext> = State::new_test(WEIGHTS, 0);
+        let target_ftt = state.total_weight() / 3;
+        let mut highway = Highway {
+            instance_id: TEST_INSTANCE_ID,
+            validators: test_validators(),
+            state,
+            active_validator: None,
+        };
+
+        let _effects =
+            highway.activate_validator(ALICE.0, ALICE_SEC.clone(), now, None, target_ftt);
+
+        let ping = Vertex::Ping(Ping::new(ALICE, now, TEST_INSTANCE_ID, &ALICE_SEC));
+        assert!(!highway.is_doppelganger_vertex(&ping));
+        let ping = Vertex::Ping(Ping::new(ALICE, later, TEST_INSTANCE_ID, &ALICE_SEC));
+        assert!(highway.is_doppelganger_vertex(&ping));
+    }
 }
