@@ -1,4 +1,5 @@
 mod block;
+mod index_panorama;
 mod panorama;
 mod params;
 mod tallies;
@@ -12,6 +13,7 @@ pub(crate) use params::Params;
 use quanta::Clock;
 pub(crate) use weight::Weight;
 
+pub(crate) use index_panorama::{IndexObservation, IndexPanorama};
 pub(crate) use panorama::{Observation, Panorama};
 pub(super) use unit::Unit;
 
@@ -475,7 +477,7 @@ impl<C: Context> State<C> {
         // Update the panorama.
         let unit = self.unit(&hash);
         let creator = unit.creator;
-        let new_obs = match (self.panorama.get(creator), unit.panorama.get(creator)) {
+        let new_obs = match (&self.panorama[creator], &unit.panorama[creator]) {
             (Observation::Faulty, _) => Observation::Faulty,
             (obs0, obs1) if obs0 == obs1 => Observation::Correct(hash),
             (Observation::None, _) => panic!("missing creator's previous unit"),
@@ -568,13 +570,11 @@ impl<C: Context> State<C> {
         self.pings[creator] = self.pings[creator].max(timestamp);
     }
 
-    /// Returns `true` if the latest timestamp we have is less than one maximum round length older
-    /// than the given timestamp.
-    ///
-    /// This is to prevent ping spam: If the incoming ping is only slightly newer than a unit or
-    /// ping we have already received, we drop it without forwarding it to our peers.
+    /// Returns `true` if the latest timestamp we have is older than the given timestamp.
     pub(crate) fn has_ping(&self, creator: ValidatorIndex, timestamp: Timestamp) -> bool {
-        self.pings.has(creator) && self.pings[creator] + self.params.max_round_length() > timestamp
+        self.pings
+            .get(creator)
+            .map_or(false, |ping_time| *ping_time >= timestamp)
     }
 
     /// Returns whether the validator's latest unit or ping is at most `PING_TIMEOUT` maximum round
@@ -728,7 +728,8 @@ impl<C: Context> State<C> {
 
     /// Returns the ancestor of the block with the given `hash`, on the specified `height`, or
     /// `None` if the block's height is lower than that.
-    pub(crate) fn find_ancestor<'a>(
+    /// NOTE: Panics if used on non-proposal hashes. For those use [`find_ancestor_unit`].
+    pub(crate) fn find_ancestor_proposal<'a>(
         &'a self,
         hash: &'a C::Hash,
         height: u64,
@@ -747,7 +748,31 @@ impl<C: Context> State<C> {
         // A block at height > 0 always has at least its parent entry in skip_idx.
         #[allow(clippy::integer_arithmetic)]
         let i = max_i.min(block.skip_idx.len() - 1);
-        self.find_ancestor(&block.skip_idx[i], height)
+        self.find_ancestor_proposal(&block.skip_idx[i], height)
+    }
+
+    /// Returns the ancestor of the unit with the given `hash`, on the specified `height`, or
+    /// `None` if the unit's height is lower than that.
+    pub(crate) fn find_ancestor_unit<'a>(
+        &'a self,
+        hash: &'a C::Hash,
+        height: u64,
+    ) -> Option<&'a C::Hash> {
+        let unit = self.unit(hash);
+        if unit.seq_number < height {
+            return None;
+        }
+        if unit.seq_number == height {
+            return Some(hash);
+        }
+        #[allow(clippy::integer_arithmetic)] // block.height > height, otherwise we returned.
+        let diff = unit.seq_number - height;
+        // We want to make the greatest step 2^i such that 2^i <= diff.
+        let max_i = log2(diff) as usize;
+        // A unit at height > 0 always has at least its parent entry in skip_idx.
+        #[allow(clippy::integer_arithmetic)]
+        let i = max_i.min(unit.skip_idx.len() - 1);
+        self.find_ancestor_unit(&unit.skip_idx[i], height)
     }
 
     /// Returns an error if `swunit` is invalid. This can be called even if the dependencies are
@@ -774,7 +799,7 @@ impl<C: Context> State<C> {
         if wunit.panorama.len() != self.validator_count() {
             return Err(UnitError::PanoramaLength(wunit.panorama.len()));
         }
-        if wunit.panorama.get(creator).is_faulty() {
+        if wunit.panorama[creator].is_faulty() {
             return Err(UnitError::FaultyCreator);
         }
         Ok(())
