@@ -7,7 +7,6 @@ use casper_node_macros::reactor;
 use futures::FutureExt;
 use tempfile::TempDir;
 use thiserror::Error;
-use tokio::time;
 
 use casper_types::ProtocolVersion;
 
@@ -20,6 +19,7 @@ use crate::{
     },
     protocol::Message,
     reactor::{Reactor as ReactorTrait, Runner},
+    testing,
     testing::{
         network::{Network, NetworkedReactor},
         ConditionCheckReactor, TestRng,
@@ -77,8 +77,9 @@ reactor!(Reactor {
             chainspec_loader.hard_reset_to_start_of_era(),
             ProtocolVersion::from_parts(1, 0, 0),
             false,
+            "test"
         );
-        deploy_acceptor = infallible DeployAcceptor(cfg.deploy_acceptor_config, &*chainspec_loader.chainspec());
+        deploy_acceptor = DeployAcceptor(cfg.deploy_acceptor_config, &*chainspec_loader.chainspec(), registry);
         deploy_fetcher = Fetcher::<Deploy>("deploy", cfg.fetcher_config, registry);
     }
 
@@ -167,7 +168,7 @@ impl Reactor {
                         ReactorEvent::DeployAcceptor(deploy_acceptor::Event::Accept {
                             deploy,
                             source: Source::Peer(sender),
-                            responder: None,
+                            maybe_responder: None,
                         }),
                     )
                 }
@@ -287,7 +288,7 @@ async fn should_fetch_from_local() {
     };
 
     // Create a random deploy.
-    let deploy = Deploy::random(&mut rng);
+    let deploy = Deploy::random_valid_native_transfer(&mut rng);
 
     // Store deploy on a node.
     let node_to_store_on = &node_ids[0];
@@ -332,7 +333,7 @@ async fn should_fetch_from_peer() {
     };
 
     // Create a random deploy.
-    let deploy = Deploy::random(&mut rng);
+    let deploy = Deploy::random_valid_native_transfer(&mut rng);
 
     // Store deploy on a node.
     let node_with_deploy = node_ids[0];
@@ -350,7 +351,14 @@ async fn should_fetch_from_peer() {
         )
         .await;
 
-    let expected_result = Some(FetchResult::FromPeer(Box::new(deploy), node_with_deploy));
+    // The deploy, while being stored, gets validated and hence updates its internal state to
+    // reflect this.  We call `is_valid()` on the deploy here to mirror that.
+    let mut validated_deploy = deploy.clone();
+    let _ = validated_deploy.is_valid();
+    let expected_result = Some(FetchResult::FromPeer(
+        Box::new(validated_deploy),
+        node_with_deploy,
+    ));
     assert_settled(
         &node_without_deploy,
         deploy_hash,
@@ -378,7 +386,7 @@ async fn should_timeout_fetch_from_peer() {
     };
 
     // Create a random deploy.
-    let deploy = Deploy::random(&mut rng);
+    let deploy = Deploy::random_valid_native_transfer(&mut rng);
     let deploy_hash = *deploy.id();
 
     let holding_node = node_ids[0];
@@ -435,10 +443,9 @@ async fn should_timeout_fetch_from_peer() {
         .await;
 
     // Advance time.
-    let secs_to_advance = Config::default().get_from_peer_timeout();
-    time::pause();
-    time::advance(Duration::from_secs(secs_to_advance + 10)).await;
-    time::resume();
+    let duration_to_advance: Duration = Config::default().get_from_peer_timeout().into();
+    let duration_to_advance = duration_to_advance + Duration::from_secs(10);
+    testing::advance_time(duration_to_advance).await;
 
     // Settle the network, allowing timeout to avoid panic.
     let expected_result = None;
