@@ -17,27 +17,28 @@ use thiserror::Error;
 use tokio::time;
 use tracing::debug;
 
+use casper_execution_engine::shared::{system_config::SystemConfig, wasm_config::WasmConfig};
 use casper_types::ProtocolVersion;
 
 use super::*;
 use crate::{
     components::{
-        contract_runtime::{self, ContractRuntime},
+        contract_runtime::{self, ContractRuntime, ContractRuntimeAnnouncement},
         deploy_acceptor::{self, DeployAcceptor},
         in_memory_network::{self, InMemoryNetwork, NetworkController},
         storage::{self, Storage},
     },
-    crypto::hash::Digest,
     effect::{
         announcements::{
-            ContractRuntimeAnnouncement, ControlAnnouncement, DeployAcceptorAnnouncement,
-            GossiperAnnouncement, NetworkAnnouncement, RpcServerAnnouncement,
+            ControlAnnouncement, DeployAcceptorAnnouncement, GossiperAnnouncement,
+            NetworkAnnouncement, RpcServerAnnouncement,
         },
         requests::{ConsensusRequest, ContractRuntimeRequest, LinearChainRequest},
         Responder,
     },
     protocol::Message as NodeMessage,
     reactor::{self, EventQueueHandle, Runner},
+    testing,
     testing::{
         network::{Network, NetworkedReactor},
         ConditionCheckReactor, TestRng,
@@ -47,6 +48,8 @@ use crate::{
     NodeRng,
 };
 
+const MAX_ASSOCIATED_KEYS: u32 = 100;
+
 /// Top-level event for the reactor.
 #[derive(Debug, From, Serialize)]
 #[must_use]
@@ -72,7 +75,7 @@ enum Event {
     #[from]
     DeployGossiperAnnouncement(#[serde(skip_serializing)] GossiperAnnouncement<Deploy>),
     #[from]
-    ContractRuntime(#[serde(skip_serializing)] contract_runtime::Event),
+    ContractRuntime(#[serde(skip_serializing)] Box<ContractRuntimeRequest>),
 }
 
 impl ReactorEvent for Event {
@@ -85,15 +88,15 @@ impl ReactorEvent for Event {
     }
 }
 
-impl From<StorageRequest> for Event {
-    fn from(request: StorageRequest) -> Self {
-        Event::Storage(storage::Event::from(request))
+impl From<ContractRuntimeRequest> for Event {
+    fn from(contract_runtime_request: ContractRuntimeRequest) -> Self {
+        Event::ContractRuntime(Box::new(contract_runtime_request))
     }
 }
 
-impl From<ContractRuntimeRequest> for Event {
-    fn from(request: ContractRuntimeRequest) -> Self {
-        Event::ContractRuntime(contract_runtime::Event::Request(Box::new(request)))
+impl From<StorageRequest> for Event {
+    fn from(request: StorageRequest) -> Self {
+        Event::Storage(storage::Event::from(request))
     }
 }
 
@@ -189,16 +192,18 @@ impl reactor::Reactor for Reactor {
             None,
             ProtocolVersion::from_parts(1, 0, 0),
             false,
+            "test",
         )
         .unwrap();
 
         let contract_runtime_config = contract_runtime::Config::default();
         let contract_runtime = ContractRuntime::new(
-            Digest::random(rng),
-            None,
             ProtocolVersion::from_parts(1, 0, 0),
-            storage_withdir,
+            storage.root_path(),
             &contract_runtime_config,
+            WasmConfig::default(),
+            SystemConfig::default(),
+            MAX_ASSOCIATED_KEYS,
             registry,
         )
         .unwrap();
@@ -206,7 +211,9 @@ impl reactor::Reactor for Reactor {
         let deploy_acceptor = DeployAcceptor::new(
             deploy_acceptor::Config::new(false),
             &Chainspec::from_resources("local"),
-        );
+            registry,
+        )
+        .unwrap();
         let deploy_gossiper = Gossiper::new_for_partial_items(
             "deploy_gossiper",
             config,
@@ -311,7 +318,7 @@ impl reactor::Reactor for Reactor {
                         Event::DeployAcceptor(deploy_acceptor::Event::Accept {
                             deploy,
                             source: Source::Peer(sender),
-                            responder: None,
+                            maybe_responder: None,
                         })
                     }
                     NodeMessage::DeployGossiper(message) => {
@@ -335,7 +342,7 @@ impl reactor::Reactor for Reactor {
                 let event = deploy_acceptor::Event::Accept {
                     deploy,
                     source: Source::<NodeId>::Client,
-                    responder,
+                    maybe_responder: responder,
                 };
                 self.dispatch_event(effect_builder, rng, Event::DeployAcceptor(event))
             }
@@ -362,9 +369,9 @@ impl reactor::Reactor for Reactor {
                 self.network.handle_event(effect_builder, rng, event),
             ),
             Event::ContractRuntime(event) => reactor::wrap_effects(
-                Event::ContractRuntime,
+                Into::into,
                 self.contract_runtime
-                    .handle_event(effect_builder, rng, event),
+                    .handle_event(effect_builder, rng, *event),
             ),
         }
     }
@@ -405,7 +412,7 @@ async fn run_gossip(rng: &mut TestRng, network_size: usize, deploy_count: usize)
 
     // Create `deploy_count` random deploys.
     let (all_deploy_hashes, mut deploys): (BTreeSet<_>, Vec<_>) = iter::repeat_with(|| {
-        let deploy = Box::new(Deploy::random(rng));
+        let deploy = Box::new(Deploy::random_valid_native_transfer(rng));
         (*deploy.id(), deploy)
     })
     .take(deploy_count)
@@ -462,7 +469,7 @@ async fn should_get_from_alternate_source() {
     let node_ids = network.add_nodes(&mut rng, NETWORK_SIZE).await;
 
     // Create random deploy.
-    let deploy = Box::new(Deploy::random(&mut rng));
+    let deploy = Box::new(Deploy::random_valid_native_transfer(&mut rng));
     let deploy_id = *deploy.id();
 
     // Give the deploy to nodes 0 and 1 to be gossiped.
@@ -504,11 +511,8 @@ async fn should_get_from_alternate_source() {
     network.settle(&mut rng, POLL_DURATION, TIMEOUT).await;
 
     // Advance time to trigger node 2's timeout causing it to request the deploy from node 1.
-    let secs_to_advance = Config::default().get_remainder_timeout_secs();
-    time::pause();
-    time::advance(Duration::from_secs(secs_to_advance)).await;
-    time::resume();
-    debug!("advanced time by {} secs", secs_to_advance);
+    let duration_to_advance = Config::default().get_remainder_timeout();
+    testing::advance_time(duration_to_advance.into()).await;
 
     // Check node 0 has the deploy stored locally.
     let deploy_held = |nodes: &HashMap<NodeId, Runner<ConditionCheckReactor<Reactor>>>| {
@@ -544,7 +548,7 @@ async fn should_timeout_gossip_response() {
         .await;
 
     // Create random deploy.
-    let deploy = Box::new(Deploy::random(&mut rng));
+    let deploy = Box::new(Deploy::random_valid_native_transfer(&mut rng));
     let deploy_id = *deploy.id();
 
     // Give the deploy to node 0 to be gossiped.
@@ -576,11 +580,8 @@ async fn should_timeout_gossip_response() {
     }
 
     // Advance time to trigger node 0's timeout causing it to gossip to the new nodes.
-    let secs_to_advance = Config::default().gossip_request_timeout_secs();
-    time::pause();
-    time::advance(Duration::from_secs(secs_to_advance)).await;
-    time::resume();
-    debug!("advanced time by {} secs", secs_to_advance);
+    let duration_to_advance = Config::default().gossip_request_timeout();
+    testing::advance_time(duration_to_advance.into()).await;
 
     // Check every node has every deploy stored locally.
     let deploy_held = |nodes: &HashMap<NodeId, Runner<ConditionCheckReactor<Reactor>>>| {
