@@ -30,7 +30,7 @@ mod peers;
 mod state;
 mod traits;
 
-use std::{convert::Infallible, fmt::Display, mem};
+use std::{cmp::Ordering, convert::Infallible, fmt::Display, mem};
 
 use datasize::DataSize;
 use prometheus::Registry;
@@ -45,21 +45,28 @@ use super::{
     Component,
 };
 use crate::{
-    effect::{EffectBuilder, EffectExt, EffectOptionExt, Effects},
+    components::contract_runtime::{
+        BlockAndExecutionEffects, ContractRuntimeAnnouncement, ExecutionPreState,
+    },
+    effect::{
+        announcements::ControlAnnouncement,
+        requests::{ContractRuntimeRequest, StorageRequest},
+        EffectBuilder, EffectExt, EffectOptionExt, Effects,
+    },
     fatal,
     types::{
-        ActivationPoint, Block, BlockByHeight, BlockHash, BlockHeader, Chainspec, FinalizedBlock,
-        TimeDiff,
+        ActivationPoint, Block, BlockByHeight, BlockHash, BlockHeader, Chainspec, Deploy,
+        FinalizedBlock, TimeDiff,
     },
     NodeRng,
 };
-pub use config::Config;
-use event::BlockByHeightResult;
-pub use event::Event;
-pub use metrics::LinearChainSyncMetrics;
-pub use peers::PeersState;
-pub use state::State;
-pub use traits::ReactorEventT;
+pub(crate) use config::Config;
+pub(crate) use event::Event;
+use event::{BlockByHeightResult, StopReason};
+pub(crate) use metrics::LinearChainSyncMetrics;
+pub(crate) use peers::PeersState;
+pub(crate) use state::State;
+pub(crate) use traits::ReactorEventT;
 
 #[derive(DataSize, Debug)]
 pub(crate) struct LinearChainSync<I> {
@@ -71,7 +78,7 @@ pub(crate) struct LinearChainSync<I> {
     /// When we download the switch block of an era immediately before the activation point,
     /// we need to shut down for an upgrade.
     next_upgrade_activation_point: Option<ActivationPoint>,
-    stop_for_upgrade: bool,
+    stop_reason: Option<StopReason>,
     /// Key for storing the linear chain sync state.
     state_key: Vec<u8>,
     /// Shortest era that is allowed with the given protocol configuration.
@@ -82,12 +89,13 @@ pub(crate) struct LinearChainSync<I> {
     started_syncing: bool,
     /// The protocol version the node is currently running with.
     protocol_version: ProtocolVersion,
+    initial_execution_pre_state: ExecutionPreState,
 }
 
 impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
     // TODO: fix this
     #[allow(clippy::too_many_arguments)]
-    pub fn new<REv, Err>(
+    pub(crate) fn new<REv, Err>(
         registry: &Registry,
         effect_builder: EffectBuilder<REv>,
         chainspec: &Chainspec,
@@ -96,6 +104,7 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
         highest_block: Option<Block>,
         after_upgrade: bool,
         next_upgrade_activation_point: Option<ActivationPoint>,
+        initial_execution_pre_state: ExecutionPreState,
         config: Config,
     ) -> Result<(Self, Effects<Event<I>>), Err>
     where
@@ -113,6 +122,7 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
                 state,
                 next_upgrade_activation_point,
                 protocol_version,
+                initial_execution_pre_state,
             )?;
             Ok((linear_chain_sync, timeout_event))
         } else {
@@ -128,6 +138,10 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
                     State::sync_trusted_hash(hash, highest_block.map(|block| block.take_header()))
                 }
                 None if after_upgrade => {
+                    info!(
+                        "No synchronization of the linear chain will be done because the node \
+                        was started right after an upgrade without a trusted hash."
+                    );
                     // Right after upgrade, no linear chain to synchronize.
                     State::Done(highest_block.map(Box::new))
                 }
@@ -139,6 +153,10 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
                         // still be trusted – i.e. it's within the unbonding period.
                         State::sync_descendants(*highest_block.hash(), highest_block, None)
                     } else {
+                        info!(
+                            "No synchronization of the linear chain will be done because there \
+                            is neither a trusted hash nor a highest block present."
+                        );
                         State::Done(None)
                     }
                 }
@@ -149,12 +167,13 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
                 state,
                 metrics: LinearChainSyncMetrics::new(registry)?,
                 next_upgrade_activation_point,
-                stop_for_upgrade: false,
+                stop_reason: None,
                 state_key,
                 shortest_era,
                 min_round_length: chainspec.highway_config.min_round_length(),
                 started_syncing: false,
                 protocol_version,
+                initial_execution_pre_state,
             };
             Ok((linear_chain_sync, timeout_event))
         }
@@ -167,6 +186,7 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
         state: State,
         next_upgrade_activation_point: Option<ActivationPoint>,
         protocol_version: ProtocolVersion,
+        initial_execution_pre_state: ExecutionPreState,
     ) -> Result<Self, prometheus::Error> {
         let state_key = create_state_key(chainspec);
         info!(?state, "reusing previous state");
@@ -175,17 +195,24 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
             chainspec.highway_config.min_round_length() * chainspec.core_config.minimum_era_height,
             chainspec.core_config.era_duration,
         );
+        if matches!(state, State::None | State::Done(_)) {
+            info!(
+                "No synchronization of the linear chain will be done because the component is \
+                already in State::Done or in State::None."
+            );
+        }
         Ok(LinearChainSync {
             peers: PeersState::new(),
             state,
             metrics: LinearChainSyncMetrics::new(registry)?,
             next_upgrade_activation_point,
-            stop_for_upgrade: false,
+            stop_reason: None,
             state_key,
             shortest_era,
             min_round_length: chainspec.highway_config.min_round_length(),
             started_syncing: false,
             protocol_version,
+            initial_execution_pre_state,
         })
     }
 
@@ -200,13 +227,18 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
     }
 
     /// Returns `true` if we have finished syncing linear chain.
-    pub fn is_synced(&self) -> bool {
+    pub(crate) fn is_synced(&self) -> bool {
         matches!(self.state, State::Done(_))
     }
 
     /// Returns `true` if we should stop for upgrade.
-    pub fn stopped_for_upgrade(&self) -> bool {
-        self.stop_for_upgrade
+    pub(crate) fn stopped_for_upgrade(&self) -> bool {
+        self.stop_reason == Some(StopReason::ForUpgrade)
+    }
+
+    /// Returns `true` if we should stop for downgrade.
+    pub(crate) fn stopped_for_downgrade(&self) -> bool {
+        self.stop_reason == Some(StopReason::ForDowngrade)
     }
 
     fn block_downloaded<REv>(
@@ -216,7 +248,7 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
         block: &Block,
     ) -> Effects<Event<I>>
     where
-        I: Send + 'static,
+        I: Send + Display + 'static,
         REv: ReactorEventT<I>,
     {
         self.peers.reset(rng);
@@ -258,8 +290,8 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
         self.state = State::Done(latest_block);
     }
 
-    /// Handles an event indicating that a linear chain block has been executed and handled by
-    /// consensus component. This is a signal that we can safely continue with the next blocks,
+    /// Handles an event indicating that a linear chain block has been executed.
+    /// This is a signal that we can safely continue with the next blocks,
     /// without worrying about timing and/or ordering issues.
     /// Returns effects that are created as a response to that event.
     fn block_handled<REv>(
@@ -269,7 +301,7 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
         block: Block,
     ) -> Effects<Event<I>>
     where
-        I: Send + 'static,
+        I: Send + Display + 'static,
         REv: ReactorEventT<I>,
     {
         let height = block.height();
@@ -407,7 +439,7 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
         effect_builder: EffectBuilder<REv>,
     ) -> Effects<Event<I>>
     where
-        I: Send + 'static,
+        I: Send + Display + 'static,
         REv: ReactorEventT<I>,
     {
         let peer = self.peers.random_unsafe();
@@ -501,7 +533,7 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
         }
         effect_builder
             .save_state(self.state_key.clone().into(), Some(self.state.clone()))
-            .event(|_| Event::Shutdown(true))
+            .event(|_| Event::Shutdown(StopReason::ForUpgrade))
     }
 
     pub(crate) fn latest_block(&self) -> Option<&Block> {
@@ -513,7 +545,7 @@ impl<I: Clone + PartialEq + 'static> LinearChainSync<I> {
         }
     }
 
-    pub fn into_maybe_latest_block_header(self) -> Option<BlockHeader> {
+    pub(crate) fn into_maybe_latest_block_header(self) -> Option<BlockHeader> {
         match self.state {
             State::SyncingTrustedHash { latest_block, .. } => latest_block.map(Block::take_header),
             State::SyncingDescendants { latest_block, .. } => Some(latest_block.take_header()),
@@ -600,7 +632,7 @@ where
                                 // trusted hash.
                                 info!(
                                     "finished synchronizing descendants of the trusted hash. \
-                                    cleaning state."
+                                     cleaning state."
                                 );
                                 self.mark_done(self.latest_block().cloned());
                                 Effects::new()
@@ -751,14 +783,52 @@ where
             Event::GetDeploysResult(fetch_result) => {
                 self.metrics.observe_get_deploys();
                 match fetch_result {
-                    event::DeploysResult::Found(block) => {
-                        let block_hash = block.hash();
-                        trace!(%block_hash, "deploys for linear chain block found");
-                        // Reset used peers so we can download next block with the full set.
-                        self.peers.reset(rng);
-                        // Execute block
-                        let finalized_block: FinalizedBlock = (*block).into();
-                        effect_builder.execute_block(finalized_block).ignore()
+                    event::DeploysResult::Found(block_to_execute) => {
+                        // Before we execute, check if the protocol version is correct - if not,
+                        // stop for an upgrade/downgrade
+                        match self
+                            .protocol_version
+                            .cmp(&block_to_execute.protocol_version())
+                        {
+                            Ordering::Less => {
+                                warn!(
+                                    block_hash = %block_to_execute.hash(),
+                                    our_version = %self.protocol_version,
+                                    block_version = %block_to_execute.protocol_version(),
+                                    "attempting to execute a block with a newer protocol version;\
+                                    shutting down to upgrade"
+                                );
+                                effect_builder
+                                    .immediately()
+                                    .event(|_| Event::Shutdown(StopReason::ForUpgrade))
+                            }
+                            Ordering::Greater => {
+                                warn!(
+                                    block_hash = %block_to_execute.hash(),
+                                    our_version = %self.protocol_version,
+                                    block_version = %block_to_execute.protocol_version(),
+                                    "attempting to execute a block with an older protocol version;\
+                                    shutting down to downgrade"
+                                );
+                                effect_builder
+                                    .immediately()
+                                    .event(|_| Event::Shutdown(StopReason::ForDowngrade))
+                            }
+                            Ordering::Equal => {
+                                let block_hash = block_to_execute.hash();
+                                let block_height = block_to_execute.height();
+                                info!(%block_hash, %block_height, "deploys for linear chain block found");
+                                // Reset used peers so we can download next block with the full set.
+                                self.peers.reset(rng);
+                                // Execute block
+                                execute_block(
+                                    effect_builder,
+                                    *block_to_execute,
+                                    self.initial_execution_pre_state.clone(),
+                                )
+                                .ignore()
+                            }
+                        }
                     }
                     event::DeploysResult::NotFound(block, peer) => {
                         let block_hash = block.hash();
@@ -808,9 +878,8 @@ where
             Event::BlockHandled(block) => {
                 let block_height = block.height();
                 let block_hash = *block.hash();
-                let effects = self.block_handled(rng, effect_builder, *block);
-                trace!(%block_height, %block_hash, "block handled");
-                effects
+                info!(%block_hash, %block_height, "executed block");
+                self.block_handled(rng, effect_builder, *block)
             }
             Event::GotUpgradeActivationPoint(next_upgrade_activation_point) => {
                 trace!(%next_upgrade_activation_point, "new activation point");
@@ -822,9 +891,9 @@ where
                 // Serialize and store state.
                 self.handle_upgrade_shutdown(effect_builder)
             }
-            Event::Shutdown(upgrade) => {
-                info!(%upgrade, "ready for shutdown");
-                self.stop_for_upgrade = upgrade;
+            Event::Shutdown(reason) => {
+                info!(?reason, "ready for shutdown");
+                self.stop_reason = Some(reason);
                 Effects::new()
             }
             Event::InitializeTimeout => {
@@ -839,7 +908,7 @@ where
     }
 }
 
-fn fetch_block_deploys<I: Clone + Send + 'static, REv>(
+fn fetch_block_deploys<I: Clone + Send + 'static + Display, REv>(
     effect_builder: EffectBuilder<REv>,
     peer: I,
     block: Block,
@@ -847,9 +916,19 @@ fn fetch_block_deploys<I: Clone + Send + 'static, REv>(
 where
     REv: ReactorEventT<I>,
 {
+    let block_height = block.height();
+    let block_hash = *block.hash();
+    info!(
+        %block_hash, %block_height, %peer,
+        "validating block from peer"
+    );
     effect_builder
         .validate_block(peer.clone(), block.clone())
         .event(move |valid| {
+            info!(
+                %block_hash, %block_height,  %peer, valid,
+                "completed validating block from peer"
+            );
             if valid {
                 Event::GetDeploysResult(DeploysResult::Found(Box::new(block)))
             } else {
@@ -967,3 +1046,110 @@ pub(crate) fn clean_linear_chain_state(
     let key = create_state_key(chainspec);
     storage.del_state_store(key)
 }
+
+async fn execute_block<REv>(
+    effect_builder: EffectBuilder<REv>,
+    block_to_execute: Block,
+    initial_execution_pre_state: ExecutionPreState,
+) where
+    REv: From<StorageRequest>
+        + From<ControlAnnouncement>
+        + From<ContractRuntimeRequest>
+        + From<ContractRuntimeAnnouncement>,
+{
+    let protocol_version = block_to_execute.protocol_version();
+    let execution_pre_state =
+        if block_to_execute.height() == initial_execution_pre_state.next_block_height() {
+            initial_execution_pre_state
+        } else {
+            match effect_builder
+                .get_block_at_height_from_storage(block_to_execute.height() - 1)
+                .await
+            {
+                None => {
+                    fatal!(
+                        effect_builder,
+                        "Could not get block at height {}",
+                        block_to_execute.height() - 1
+                    )
+                    .await;
+                    return;
+                }
+                Some(parent_block) => ExecutionPreState::from(parent_block.header()),
+            }
+        };
+    let finalized_block = FinalizedBlock::from(block_to_execute);
+
+    // Get the deploy hashes for the block.
+    let deploy_hashes = finalized_block
+        .deploy_hashes()
+        .iter()
+        .copied()
+        .collect::<Vec<_>>();
+
+    // Get all deploys in order they appear in the finalized block.
+    let mut deploys: Vec<Deploy> = Vec::with_capacity(deploy_hashes.len());
+    for maybe_deploy in effect_builder.get_deploys_from_storage(deploy_hashes).await {
+        if let Some(deploy) = maybe_deploy {
+            deploys.push(deploy)
+        } else {
+            fatal!(
+                effect_builder,
+                "Could not fetch deploys for finalized block: {:?}",
+                finalized_block
+            )
+            .await;
+            return;
+        }
+    }
+
+    // Get the transfer hashes for the block.
+    let transfer_hashes = finalized_block
+        .transfer_hashes()
+        .iter()
+        .copied()
+        .collect::<Vec<_>>();
+
+    // Get all deploys in order they appear in the finalized block.
+    let mut transfers: Vec<Deploy> = Vec::with_capacity(transfer_hashes.len());
+    for maybe_transfer in effect_builder
+        .get_deploys_from_storage(transfer_hashes)
+        .await
+    {
+        if let Some(transfer) = maybe_transfer {
+            transfers.push(transfer)
+        } else {
+            fatal!(
+                effect_builder,
+                "Could not fetch deploys for finalized block: {:?}",
+                finalized_block
+            )
+            .await;
+            return;
+        }
+    }
+
+    let BlockAndExecutionEffects {
+        block,
+        execution_results,
+        ..
+    } = match effect_builder
+        .execute_finalized_block(
+            protocol_version,
+            execution_pre_state,
+            finalized_block,
+            deploys,
+            transfers,
+        )
+        .await
+    {
+        Ok(child_block) => child_block,
+        Err(error) => {
+            fatal!(effect_builder, "Fatal error: {}", error).await;
+            return;
+        }
+    };
+    effect_builder
+        .announce_linear_chain_block(block, execution_results)
+        .await;
+}
