@@ -59,6 +59,8 @@ pub(super) async fn run<REv: ReactorEventT>(
     let rpc_get_state_root_hash =
         rpcs::chain::GetStateRootHash::create_filter(effect_builder, api_version);
     let rpc_get_item = rpcs::state::GetItem::create_filter(effect_builder, api_version);
+    let rpc_query_global_state =
+        rpcs::state::QueryGlobalState::create_filter(effect_builder, api_version);
     let rpc_get_balance = rpcs::state::GetBalance::create_filter(effect_builder, api_version);
     let rpc_get_account_info =
         rpcs::state::GetAccountInfo::create_filter(effect_builder, api_version);
@@ -69,6 +71,9 @@ pub(super) async fn run<REv: ReactorEventT>(
         rpcs::chain::GetEraInfoBySwitchBlock::create_filter(effect_builder, api_version);
     let rpc_get_auction_info =
         rpcs::state::GetAuctionInfo::create_filter(effect_builder, api_version);
+    let rpc_get_trie = rpcs::state::GetTrie::create_filter(effect_builder, api_version);
+    let rpcs_get_validator_changes =
+        rpcs::info::GetValidatorChanges::create_filter(effect_builder, api_version);
     let rpc_get_rpcs = rpcs::docs::ListRpcs::create_filter(effect_builder, api_version);
     let rpc_get_dictionary_item =
         rpcs::state::GetDictionaryItem::create_filter(effect_builder, api_version);
@@ -88,27 +93,36 @@ pub(super) async fn run<REv: ReactorEventT>(
         Ok::<_, Rejection>(error_response)
     });
 
+    let service_routes = rpc_put_deploy
+        .or(rpc_get_block)
+        .or(rpc_get_block_transfers)
+        .or(rpc_get_state_root_hash)
+        .or(rpc_get_item)
+        .or(rpc_get_balance)
+        .or(rpc_get_deploy)
+        .or(rpc_get_peers)
+        .or(rpc_get_status)
+        .or(rpc_get_era_info)
+        .or(rpc_get_auction_info)
+        .or(rpc_get_account_info)
+        .or(rpcs_get_validator_changes)
+        .or(rpc_get_rpcs)
+        .or(rpc_get_dictionary_item)
+        .or(rpc_get_trie)
+        .or(rpc_query_global_state)
+        .or(unknown_method)
+        .or(parse_failure);
+
+    // Supports content negotiation for gzip responses. This is an interim fix until
+    // https://github.com/seanmonstar/warp/pull/513 moves forward.
+    let service_routes_gzip = warp::header::exact("accept-encoding", "gzip")
+        .and(service_routes.clone())
+        .with(warp::compression::gzip());
+
     // TODO - we can't catch cases where we should return `warp_json_rpc::Error::INVALID_REQUEST`
     //        (i.e. where the request is JSON, but not valid JSON-RPC).  This will require an
     //        update to or move away from warp_json_rpc.
-    let service = warp_json_rpc::service(
-        rpc_put_deploy
-            .or(rpc_get_block)
-            .or(rpc_get_block_transfers)
-            .or(rpc_get_state_root_hash)
-            .or(rpc_get_item)
-            .or(rpc_get_balance)
-            .or(rpc_get_deploy)
-            .or(rpc_get_peers)
-            .or(rpc_get_status)
-            .or(rpc_get_era_info)
-            .or(rpc_get_auction_info)
-            .or(rpc_get_account_info)
-            .or(rpc_get_rpcs)
-            .or(rpc_get_dictionary_item)
-            .or(unknown_method)
-            .or(parse_failure),
-    );
+    let service = warp_json_rpc::service(service_routes_gzip.or(service_routes));
 
     // Start the server, passing a oneshot receiver to allow the server to be shut down gracefully.
     let make_svc =
