@@ -30,25 +30,25 @@ use crate::{
     effect::EffectBuilder,
     rpcs::{
         chain::GetEraInfoBySwitchBlock,
-        state::{GetAccountInfo, GetDictionaryItem},
+        info::GetValidatorChanges,
+        state::{GetAccountInfo, GetDictionaryItem, QueryGlobalState},
     },
 };
 
 pub(crate) const DOCS_EXAMPLE_PROTOCOL_VERSION: ProtocolVersion =
-    ProtocolVersion::from_parts(1, 3, 4);
+    ProtocolVersion::from_parts(1, 4, 1);
 
 const DEFINITIONS_PATH: &str = "#/components/schemas/";
 
 // As per https://spec.open-rpc.org/#service-discovery-method.
 pub(crate) static OPEN_RPC_SCHEMA: Lazy<OpenRpcSchema> = Lazy::new(|| {
     let contact = OpenRpcContactField {
-        name: "Casper".to_string(),
-        url: "https://casper.network".to_string(),
+        name: "CasperLabs".to_string(),
+        url: "https://casperlabs.io".to_string(),
     };
     let license = OpenRpcLicenseField {
-        name: "Apache License Version 2.0".to_string(),
-        url: "https://raw.githubusercontent.com/casper-network/casper-node/master/LICENSE"
-            .to_string(),
+        name: "CasperLabs Open Source License Version 1.0".to_string(),
+        url: "https://raw.githubusercontent.com/CasperLabs/casper-node/master/LICENSE".to_string(),
     };
     let info = OpenRpcInfoField {
         version: DOCS_EXAMPLE_PROTOCOL_VERSION.to_string(),
@@ -78,8 +78,13 @@ pub(crate) static OPEN_RPC_SCHEMA: Lazy<OpenRpcSchema> = Lazy::new(|| {
     schema.push_with_params::<GetDeploy>("returns a Deploy from the network");
     schema.push_with_params::<GetAccountInfo>("returns an Account from the network");
     schema.push_with_params::<GetDictionaryItem>("returns an item from a Dictionary");
+    schema.push_with_params::<QueryGlobalState>(
+        "a query to global state using either a Block hash or state root hash",
+    );
     schema.push_without_params::<GetPeers>("returns a list of peers connected to the node");
     schema.push_without_params::<GetStatus>("returns the current status of the node");
+    schema
+        .push_without_params::<GetValidatorChanges>("returns status changes of active validators");
     schema.push_with_optional_params::<GetBlock>("returns a Block from the network");
     schema.push_with_optional_params::<GetBlockTransfers>(
         "returns all transfers for a Block from the network",
@@ -87,7 +92,7 @@ pub(crate) static OPEN_RPC_SCHEMA: Lazy<OpenRpcSchema> = Lazy::new(|| {
     schema.push_with_optional_params::<GetStateRootHash>(
         "returns a state root hash at a given Block",
     );
-    schema.push_with_params::<GetItem>("returns a stored value from the network");
+    schema.push_with_params::<GetItem>("returns a stored value from the network. This RPC is deprecated, use `query_global_state` instead.");
     schema.push_with_params::<GetBalance>("returns a purse's balance from the network");
     schema.push_with_optional_params::<GetEraInfoBySwitchBlock>(
         "returns an EraInfo from the network",
@@ -445,7 +450,6 @@ impl RpcWithoutParamsExt for ListRpcs {
         async move { Ok(response_builder.success(ListRpcsResult::doc_example().clone())?) }.boxed()
     }
 }
-
 #[cfg(test)]
 mod tests {
     use crate::{types::Chainspec, utils::Loadable};
