@@ -14,12 +14,12 @@ use serde::{Deserialize, Serialize};
 use tracing::{error, info};
 use warp_json_rpc::Builder;
 
-use casper_execution_engine::{
-    core::engine_state::{BalanceResult, GetBidsResult, QueryResult},
-    shared::stored_value::StoredValue as DomainStoredValue,
-};
+use casper_execution_engine::core::engine_state::{BalanceResult, GetBidsResult, QueryResult};
+use casper_hashing::Digest;
 use casper_types::{
-    bytesrepr::ToBytes, CLValue, Key, ProtocolVersion, PublicKey, SecretKey, URef, U512,
+    bytesrepr::{Bytes, ToBytes},
+    CLValue, Key, ProtocolVersion, PublicKey, SecretKey, StoredValue as DomainStoredValue, URef,
+    U512,
 };
 
 use super::{
@@ -28,7 +28,6 @@ use super::{
 };
 use crate::{
     components::rpc_server::rpcs::RpcWithOptionalParams,
-    crypto::hash::Digest,
     effect::EffectBuilder,
     reactor::QueueKind,
     rpcs::{
@@ -38,7 +37,7 @@ use crate::{
     },
     types::{
         json_compatibility::{Account as JsonAccount, AuctionState, StoredValue},
-        Block,
+        Block, BlockHash, JsonBlockHeader,
     },
 };
 
@@ -100,6 +99,26 @@ static GET_DICTIONARY_ITEM_RESULT: Lazy<GetDictionaryItemResult> =
         stored_value: StoredValue::CLValue(CLValue::from_t(1u64).unwrap()),
         merkle_proof: MERKLE_PROOF.clone(),
     });
+static QUERY_GLOBAL_STATE_PARAMS: Lazy<QueryGlobalStateParams> =
+    Lazy::new(|| QueryGlobalStateParams {
+        state_identifier: GlobalStateIdentifier::BlockHash(*Block::doc_example().hash()),
+        key: "deploy-af684263911154d26fa05be9963171802801a0b6aff8f199b7391eacb8edc9e1".to_string(),
+        path: vec![],
+    });
+static QUERY_GLOBAL_STATE_RESULT: Lazy<QueryGlobalStateResult> =
+    Lazy::new(|| QueryGlobalStateResult {
+        api_version: DOCS_EXAMPLE_PROTOCOL_VERSION,
+        block_header: Some(JsonBlockHeader::doc_example().clone()),
+        stored_value: StoredValue::Account(JsonAccount::doc_example().clone()),
+        merkle_proof: MERKLE_PROOF.clone(),
+    });
+static GET_TRIE_PARAMS: Lazy<GetTrieParams> = Lazy::new(|| GetTrieParams {
+    trie_key: *Block::doc_example().header().state_root_hash(),
+});
+static GET_TRIE_RESULT: Lazy<GetTrieResult> = Lazy::new(|| GetTrieResult {
+    api_version: DOCS_EXAMPLE_PROTOCOL_VERSION,
+    maybe_trie_bytes: None,
+});
 
 /// Params for "state_get_item" RPC request.
 #[derive(Serialize, Deserialize, Debug, JsonSchema)]
@@ -418,11 +437,26 @@ impl RpcWithOptionalParamsExt for GetAuctionInfo {
                 )
                 .await;
 
-            let maybe_bids = if let Ok(GetBidsResult::Success { bids, .. }) = get_bids_result {
-                Some(bids)
-            } else {
-                None
+            let bids = match get_bids_result {
+                Ok(get_bids_result) => match get_bids_result {
+                    GetBidsResult::RootNotFound => {
+                        error!(block_hash=?block.hash(), ?state_root_hash, "failed to get bids");
+                        return Ok(response_builder.error(warp_json_rpc::Error::custom(
+                            ErrorCode::InternalError as i64,
+                            format!("get-auction-info failed to get bids at block={:?}", block.hash()),
+                        ))?);
+                    }
+                    GetBidsResult::Success { bids } => bids,
+                },
+                Err(err) => {
+                    error!(block_hash=?block.hash(), ?state_root_hash, ?err, "failed to get bids");
+                    return Ok(response_builder.error(warp_json_rpc::Error::custom(
+                        ErrorCode::InternalError as i64,
+                        format!("get-auction-info failed to get bids at block={:?}", block.hash()),
+                    ))?);
+                }
             };
+
             let era_validators_result = effect_builder
                 .make_request(
                     |responder| RpcRequest::QueryEraValidators {
@@ -434,10 +468,19 @@ impl RpcWithOptionalParamsExt for GetAuctionInfo {
                 )
                 .await;
 
-            let era_validators = era_validators_result.ok();
+            let era_validators = match era_validators_result {
+                Ok(validators) => validators,
+                Err(err) => {
+                    error!(block_hash=?block.hash(), ?state_root_hash, ?err, "failed to get era validators");
+                    return Ok(response_builder.error(warp_json_rpc::Error::custom(
+                        ErrorCode::InternalError as i64,
+                        format!("get-auction-info failed to get validators at block={:?}", block.hash()),
+                    ))?);
+                }
+            };
 
             let auction_state =
-                AuctionState::new(state_root_hash, block_height, era_validators, maybe_bids);
+                AuctionState::new(state_root_hash, block_height, era_validators, bids);
 
             let result = Self::ResponseResult {
                 api_version,
@@ -715,7 +758,7 @@ impl DocExample for GetDictionaryItemResult {
     }
 }
 
-/// "state_get_dictionary" RPC.
+/// "state_get_dictionary_item" RPC.
 pub struct GetDictionaryItem {}
 
 impl RpcWithParams for GetDictionaryItem {
@@ -830,3 +873,230 @@ impl RpcWithParamsExt for GetDictionaryItem {
         .boxed()
     }
 }
+
+/// Identifier for possible ways to query Global State
+#[derive(Serialize, Deserialize, Debug, JsonSchema, Clone)]
+#[serde(deny_unknown_fields)]
+pub enum GlobalStateIdentifier {
+    /// Query using a block hash.
+    BlockHash(BlockHash),
+    /// Query using the state root hash.
+    StateRootHash(Digest),
+}
+
+/// Params for "query_global_state" RPC
+#[derive(Serialize, Deserialize, Debug, JsonSchema)]
+#[serde(deny_unknown_fields)]
+pub struct QueryGlobalStateParams {
+    /// The identifier used for the query.
+    pub state_identifier: GlobalStateIdentifier,
+    /// `casper_types::Key` as formatted string.
+    pub key: String,
+    /// The path components starting from the key as base.
+    #[serde(default)]
+    pub path: Vec<String>,
+}
+
+impl DocExample for QueryGlobalStateParams {
+    fn doc_example() -> &'static Self {
+        &*QUERY_GLOBAL_STATE_PARAMS
+    }
+}
+
+/// Result for "query_global_state" RPC response.
+#[derive(Serialize, Deserialize, Debug, JsonSchema)]
+#[serde(deny_unknown_fields)]
+pub struct QueryGlobalStateResult {
+    /// The RPC API version.
+    #[schemars(with = "String")]
+    pub api_version: ProtocolVersion,
+    /// The block header if a Block hash was provided.
+    pub block_header: Option<JsonBlockHeader>,
+    /// The stored value.
+    pub stored_value: StoredValue,
+    /// The merkle proof.
+    pub merkle_proof: String,
+}
+
+impl DocExample for QueryGlobalStateResult {
+    fn doc_example() -> &'static Self {
+        &*QUERY_GLOBAL_STATE_RESULT
+    }
+}
+
+/// "query_global_state" RPC
+pub struct QueryGlobalState {}
+
+impl RpcWithParams for QueryGlobalState {
+    const METHOD: &'static str = "query_global_state";
+    type RequestParams = QueryGlobalStateParams;
+    type ResponseResult = QueryGlobalStateResult;
+}
+
+impl RpcWithParamsExt for QueryGlobalState {
+    fn handle_request<REv: ReactorEventT>(
+        effect_builder: EffectBuilder<REv>,
+        response_builder: Builder,
+        params: Self::RequestParams,
+        api_version: ProtocolVersion,
+    ) -> BoxFuture<'static, Result<Response<Body>, Error>> {
+        async move {
+            let (state_root_hash, maybe_block_header) = match params.state_identifier {
+                GlobalStateIdentifier::BlockHash(block_hash) => {
+                    match effect_builder
+                        .get_block_header_from_storage(block_hash)
+                        .await
+                    {
+                        Some(header) => {
+                            let json_block_header = JsonBlockHeader::from(header.clone());
+                            (*header.state_root_hash(), Some(json_block_header))
+                        }
+                        None => {
+                            let error_msg =
+                                "query_global_state failed to retrieve specified block header"
+                                    .to_string();
+                            return Ok(response_builder.error(warp_json_rpc::Error::custom(
+                                ErrorCode::NoSuchBlock as i64,
+                                error_msg,
+                            ))?);
+                        }
+                    }
+                }
+                GlobalStateIdentifier::StateRootHash(state_root_hash) => (state_root_hash, None),
+            };
+
+            let base_key = match Key::from_formatted_str(&params.key)
+                .map_err(|error| format!("failed to parse key: {}", error))
+            {
+                Ok(key) => key,
+                Err(error_msg) => {
+                    info!("{}", error_msg);
+                    return Ok(response_builder.error(warp_json_rpc::Error::custom(
+                        ErrorCode::ParseQueryKey as i64,
+                        error_msg,
+                    ))?);
+                }
+            };
+
+            let query_result = effect_builder
+                .make_request(
+                    |responder| RpcRequest::QueryGlobalState {
+                        state_root_hash,
+                        base_key,
+                        path: params.path,
+                        responder,
+                    },
+                    QueueKind::Api,
+                )
+                .await;
+
+            let (stored_value, proof_bytes) = match common::extract_query_result(query_result) {
+                Ok(tuple) => tuple,
+                Err((error_code, error_msg)) => {
+                    info!("{}", error_msg);
+                    return Ok(response_builder
+                        .error(warp_json_rpc::Error::custom(error_code as i64, error_msg))?);
+                }
+            };
+
+            let result = Self::ResponseResult {
+                api_version,
+                block_header: maybe_block_header,
+                stored_value,
+                merkle_proof: hex::encode(proof_bytes),
+            };
+
+            Ok(response_builder.success(result)?)
+        }
+        .boxed()
+    }
+}
+
+/// Parameters for "state_get_trie" RPC request.
+#[derive(Serialize, Deserialize, Debug, JsonSchema)]
+pub struct GetTrieParams {
+    /// A trie key.
+    pub trie_key: Digest,
+}
+
+impl DocExample for GetTrieParams {
+    fn doc_example() -> &'static Self {
+        &*GET_TRIE_PARAMS
+    }
+}
+
+/// Result for "state_get_trie" RPC response.
+#[derive(Serialize, Deserialize, Debug, JsonSchema)]
+#[serde(deny_unknown_fields)]
+pub struct GetTrieResult {
+    /// The RPC API version.
+    #[schemars(with = "String")]
+    pub api_version: ProtocolVersion,
+    /// A list of keys read under the specified prefix.
+    #[schemars(
+        with = "Option<String>",
+        description = "A trie from global state storage, bytesrepr serialized and hex-encoded."
+    )]
+    pub maybe_trie_bytes: Option<Bytes>,
+}
+
+impl DocExample for GetTrieResult {
+    fn doc_example() -> &'static Self {
+        &*GET_TRIE_RESULT
+    }
+}
+
+/// `state_get_trie` RPC.
+pub struct GetTrie {}
+
+impl RpcWithParams for GetTrie {
+    const METHOD: &'static str = "state_get_trie";
+    type RequestParams = GetTrieParams;
+    type ResponseResult = GetTrieResult;
+}
+
+impl RpcWithParamsExt for GetTrie {
+    fn handle_request<REv: ReactorEventT>(
+        effect_builder: EffectBuilder<REv>,
+        response_builder: Builder,
+        params: Self::RequestParams,
+        api_version: ProtocolVersion,
+    ) -> BoxFuture<'static, Result<Response<Body>, Error>> {
+        async move {
+            let trie_key = params.trie_key;
+
+            let ee_trie = match effect_builder.get_trie(trie_key).await {
+                Ok(Some(trie)) => trie,
+                Ok(None) => {
+                    return Ok(response_builder.success(Self::ResponseResult {
+                        api_version,
+                        maybe_trie_bytes: None,
+                    })?)
+                }
+                Err(error) => {
+                    error!(?error, "failed to get trie");
+                    return Ok(response_builder.error(warp_json_rpc::Error::custom(
+                        ErrorCode::FailedToGetTrie as i64,
+                        format!("failed to get trie: {:?}", error),
+                    ))?);
+                }
+            };
+
+            let trie_bytes = match ee_trie.to_bytes() {
+                Ok(bytes) => bytes,
+                Err(error) => {
+                    error!(?error, "failed to serialize trie");
+                    return Ok(response_builder.error(warp_json_rpc::Error::INTERNAL_ERROR)?);
+                }
+            };
+
+            let result = Self::ResponseResult {
+                api_version,
+                maybe_trie_bytes: Some(trie_bytes.into()),
+            };
+
+            Ok(response_builder.success(result)?)
+        }
+        .boxed()
+    }
+}
