@@ -5,6 +5,7 @@ use std::{
 };
 
 use casper_types::{ProtocolVersion, PublicKey, SecretKey, Signature};
+use datasize::DataSize;
 use serde::{de::DeserializeOwned, Deserialize, Serialize};
 
 use crate::crypto;
@@ -18,7 +19,7 @@ fn default_protocol_version() -> ProtocolVersion {
 }
 
 #[derive(Clone, Debug, Deserialize, Serialize)]
-pub enum Message<P> {
+pub(crate) enum Message<P> {
     Handshake {
         /// Network we are connected to.
         network_name: String,
@@ -46,10 +47,10 @@ impl<P: Payload> Message<P> {
 
     /// Returns the incoming resource estimate of the payload.
     #[inline]
-    pub(super) fn payload_incoming_resource_estimate(&self) -> u32 {
+    pub(super) fn payload_incoming_resource_estimate(&self, weights: &PayloadWeights) -> u32 {
         match self {
             Message::Handshake { .. } => 0,
-            Message::Payload(payload) => payload.incoming_resource_estimate(),
+            Message::Payload(payload) => payload.incoming_resource_estimate(weights),
         }
     }
 }
@@ -77,7 +78,7 @@ impl ConsensusKeyPair {
 
 /// Certificate used to indicate that the peer is a validator using the specified public key.
 #[derive(Clone, Debug, Deserialize, Serialize)]
-pub struct ConsensusCertificate {
+pub(crate) struct ConsensusCertificate {
     public_key: PublicKey,
     signature: Signature,
 }
@@ -133,7 +134,7 @@ impl<P: Display> Display for Message<P> {
 
 /// A classification system for networking messages.
 #[derive(Copy, Clone, Debug)]
-pub enum MessageKind {
+pub(crate) enum MessageKind {
     /// Non-payload messages, like handshakes.
     Protocol,
     /// Messages directly related to consensus.
@@ -168,18 +169,31 @@ impl Display for MessageKind {
 ///
 /// Payloads are what is transferred across the network outside of control messages from the
 /// networking component itself.
-pub trait Payload:
+pub(crate) trait Payload:
     Serialize + DeserializeOwned + Clone + Debug + Display + Send + Sync + 'static
 {
     /// Classifies the payload based on its contents.
     fn classify(&self) -> MessageKind;
 
     /// The penalty for resource usage of a message to be applied when processed as incoming.
-    fn incoming_resource_estimate(&self) -> u32 {
+    fn incoming_resource_estimate(&self, _weights: &PayloadWeights) -> u32 {
         0
     }
 }
 
+/// A generic configuration for payload weights.
+///
+/// Implementors of `Payload` are free to interpret this as they see fit.
+///
+/// The default implementation sets all weights to zero.
+#[derive(DataSize, Debug, Default, Clone, Deserialize, Serialize)]
+pub struct PayloadWeights {
+    /// Weight to attach to consensus traffic.
+    pub consensus: u32,
+    /// Weight to attach to deploy requests.
+    pub deploy_requests: u32,
+}
+
 #[cfg(test)]
 // We use a variety of weird names in these tests.
 #[allow(non_camel_case_types)]
@@ -198,7 +212,7 @@ mod tests {
     /// Note that the message itself may go out of sync over time as `protocol::Message` changes.
     /// The test further below ensures that the handshake is accurate in the meantime.
     #[derive(Clone, Debug, Deserialize, Serialize)]
-    pub enum V1_0_0_Message {
+    pub(crate) enum V1_0_0_Message {
         Handshake {
             /// Network we are connected to.
             network_name: String,
