@@ -58,8 +58,8 @@
 //! A request **must** have a `Responder` field, which a handler of a request **must** call at
 //! some point. Failing to do so will result in a resource leak.
 
-pub mod announcements;
-pub mod requests;
+pub(crate) mod announcements;
+pub(crate) mod requests;
 
 use std::{
     any::type_name,
@@ -74,7 +74,7 @@ use std::{
 use datasize::DataSize;
 use futures::{channel::oneshot, future::BoxFuture, FutureExt};
 use once_cell::sync::Lazy;
-use serde::{de::DeserializeOwned, Serialize};
+use serde::Serialize;
 use smallvec::{smallvec, SmallVec};
 use tokio::{sync::Semaphore, time};
 use tracing::error;
@@ -85,33 +85,28 @@ use casper_execution_engine::{
     core::engine_state::{
         self,
         era_validators::GetEraValidatorsError,
-        execution_effect::ExecutionEffect,
-        genesis::GenesisResult,
-        step::{StepRequest, StepResult},
-        upgrade::{UpgradeConfig, UpgradeResult},
+        genesis::GenesisSuccess,
+        upgrade::{UpgradeConfig, UpgradeSuccess},
         BalanceRequest, BalanceResult, GetBidsRequest, GetBidsResult, QueryRequest, QueryResult,
-        MAX_PAYMENT,
     },
-    shared::{newtypes::Blake2bHash, stored_value::StoredValue},
-    storage::{protocol_data::ProtocolData, trie::Trie},
+    storage::trie::Trie,
 };
+use casper_hashing::Digest;
 use casper_types::{
-    system::auction::EraValidators, EraId, ExecutionResult, Key, ProtocolVersion, PublicKey,
-    Transfer, U512,
+    account::Account, system::auction::EraValidators, Contract, ContractPackage, EraId,
+    ExecutionResult, Key, ProtocolVersion, PublicKey, StoredValue, Transfer, URef, U512,
 };
 
 use crate::{
     components::{
         block_validator::ValidatingBlock,
         chainspec_loader::{CurrentRunInfo, NextUpgrade},
-        consensus::{BlockContext, ClContext},
+        consensus::{BlockContext, ClContext, ValidatorChange},
         contract_runtime::EraValidatorsRequest,
         deploy_acceptor,
         fetcher::FetchResult,
         small_network::GossipedAddress,
     },
-    crypto::hash::Digest,
-    effect::requests::LinearChainRequest,
     reactor::{EventQueueHandle, QueueKind},
     types::{
         Block, BlockByHeight, BlockHash, BlockHeader, BlockPayload, BlockSignatures, Chainspec,
@@ -121,9 +116,9 @@ use crate::{
     utils::Source,
 };
 use announcements::{
-    ChainspecLoaderAnnouncement, ConsensusAnnouncement, ContractRuntimeAnnouncement,
-    ControlAnnouncement, DeployAcceptorAnnouncement, GossiperAnnouncement, LinearChainAnnouncement,
-    NetworkAnnouncement, RpcServerAnnouncement,
+    ChainspecLoaderAnnouncement, ConsensusAnnouncement, ControlAnnouncement,
+    DeployAcceptorAnnouncement, GossiperAnnouncement, LinearChainAnnouncement, NetworkAnnouncement,
+    RpcServerAnnouncement,
 };
 use requests::{
     BlockPayloadRequest, BlockProposerRequest, BlockValidationRequest, ChainspecLoaderRequest,
@@ -131,16 +126,19 @@ use requests::{
     NetworkRequest, StateStoreRequest, StorageRequest,
 };
 
-use self::announcements::BlocklistAnnouncement;
+use self::announcements::{BlockProposerAnnouncement, BlocklistAnnouncement};
+use crate::components::contract_runtime::{
+    BlockAndExecutionEffects, BlockExecutionError, ContractRuntimeAnnouncement, ExecutionPreState,
+};
 
 /// A resource that will never be available, thus trying to acquire it will wait forever.
 static UNOBTAINABLE: Lazy<Semaphore> = Lazy::new(|| Semaphore::new(0));
 
 /// A pinned, boxed future that produces one or more events.
-pub type Effect<Ev> = BoxFuture<'static, Multiple<Ev>>;
+pub(crate) type Effect<Ev> = BoxFuture<'static, Multiple<Ev>>;
 
 /// Multiple effects in a container.
-pub type Effects<Ev> = Multiple<Effect<Ev>>;
+pub(crate) type Effects<Ev> = Multiple<Effect<Ev>>;
 
 /// A small collection of rarely more than two items.
 ///
@@ -148,12 +146,12 @@ pub type Effects<Ev> = Multiple<Effect<Ev>>;
 /// size of two items is chosen because one item is the most common use case, and large items are
 /// typically boxed. In the latter case two pointers and one enum variant discriminator is almost
 /// the same size as an empty vec, which is two pointers.
-pub type Multiple<T> = SmallVec<[T; 2]>;
+pub(crate) type Multiple<T> = SmallVec<[T; 2]>;
 
 /// A responder satisfying a request.
 #[must_use]
 #[derive(DataSize)]
-pub struct Responder<T>(Option<oneshot::Sender<T>>);
+pub(crate) struct Responder<T>(Option<oneshot::Sender<T>>);
 
 impl<T: 'static + Send> Responder<T> {
     /// Creates a new `Responder`.
@@ -173,15 +171,24 @@ impl<T: 'static + Send> Responder<T> {
     }
 }
 
-impl<T> Responder<T> {
+impl<T> Responder<T>
+where
+    T: Debug,
+{
     /// Send `data` to the origin of the request.
-    pub async fn respond(mut self, data: T) {
+    pub(crate) async fn respond(mut self, data: T) {
         if let Some(sender) = self.0.take() {
-            if sender.send(data).is_err() {
-                error!("could not send response to request down oneshot channel");
+            if let Err(data) = sender.send(data) {
+                error!(
+                    ?data,
+                    "could not send response to request down oneshot channel"
+                );
             }
         } else {
-            error!("tried to send a value down a responder channel, but it was already used");
+            error!(
+                ?data,
+                "tried to send a value down a responder channel, but it was already used"
+            );
         }
     }
 }
@@ -203,9 +210,9 @@ impl<T> Drop for Responder<T> {
         if self.0.is_some() {
             // This is usually a very serious error, as another component will now be stuck.
             error!(
-                "{} dropped without being responded to --- \
-                 this is always a bug and will likely cause another component to be stuck!",
-                self
+                responder=?self,
+                "dropped without being responded to --- \
+                 this is always a bug and will likely cause another component to be stuck!"
             );
         }
     }
@@ -221,7 +228,7 @@ impl<T> Serialize for Responder<T> {
 }
 
 /// Effect extension for futures, used to convert futures into actual effects.
-pub trait EffectExt: Future + Send {
+pub(crate) trait EffectExt: Future + Send {
     /// Finalizes a future into an effect that returns a single event.
     ///
     /// The function `f` is used to translate the returned value from an effect into an event.
@@ -248,7 +255,7 @@ pub trait EffectExt: Future + Send {
 
 /// Effect extension for futures, used to convert futures returning a `Result` into two different
 /// effects.
-pub trait EffectResultExt {
+pub(crate) trait EffectResultExt {
     /// The type the future will return if `Ok`.
     type Value;
     /// The type the future will return if `Err`.
@@ -267,7 +274,7 @@ pub trait EffectResultExt {
 
 /// Effect extension for futures, used to convert futures returning an `Option` into two different
 /// effects.
-pub trait EffectOptionExt {
+pub(crate) trait EffectOptionExt {
     /// The type the future will return if `Some`.
     type Value;
 
@@ -379,7 +386,7 @@ where
 /// Provides methods allowing the creation of effects which need to be scheduled
 /// on the reactor's event queue, without giving direct access to this queue.
 #[derive(Debug)]
-pub struct EffectBuilder<REv: 'static>(EventQueueHandle<REv>);
+pub(crate) struct EffectBuilder<REv: 'static>(EventQueueHandle<REv>);
 
 // Implement `Clone` and `Copy` manually, as `derive` will make it depend on `REv` otherwise.
 impl<REv> Clone for EffectBuilder<REv> {
@@ -392,13 +399,21 @@ impl<REv> Copy for EffectBuilder<REv> {}
 
 impl<REv> EffectBuilder<REv> {
     /// Creates a new effect builder.
-    pub fn new(event_queue_handle: EventQueueHandle<REv>) -> Self {
+    pub(crate) fn new(event_queue_handle: EventQueueHandle<REv>) -> Self {
         EffectBuilder(event_queue_handle)
     }
 
+    /// Schedules a regular event.
+    pub(crate) async fn schedule_regular<E>(self, event: E)
+    where
+        REv: From<E>,
+    {
+        self.0.schedule(event, QueueKind::Regular).await
+    }
+
     /// Extract the event queue handle out of the effect builder.
     #[cfg(test)]
-    pub fn into_inner(self) -> EventQueueHandle<REv> {
+    pub(crate) fn into_inner(self) -> EventQueueHandle<REv> {
         self.0
     }
 
@@ -433,8 +448,8 @@ impl<REv> EffectBuilder<REv> {
             Err(err) => {
                 // The channel should never be closed, ever. If it is, we pretend nothing happened
                 // though, instead of crashing.
-                error!(%err, ?queue_kind, "request for {} channel closed, this may be a bug? \
-                       check if a component is stuck from now on ", type_name::<T>());
+                error!(%err, ?queue_kind, channel=?type_name::<T>(), "request for channel closed, this may be a bug? \
+                       check if a component is stuck from now on ");
 
                 // We cannot produce any value to satisfy the request, so we just abandon this task
                 // by waiting on a resource we can never acquire.
@@ -450,7 +465,7 @@ impl<REv> EffectBuilder<REv> {
     /// "do nothing", as it will still cause a task to be spawned.
     #[inline(always)]
     #[allow(clippy::manual_async_fn)]
-    pub fn immediately(self) -> impl Future<Output = ()> + Send {
+    pub(crate) fn immediately(self) -> impl Future<Output = ()> + Send {
         // Note: This function is implemented manually without `async` sugar because the `Send`
         // inference seems to not work in all cases otherwise.
         async {}
@@ -462,7 +477,7 @@ impl<REv> EffectBuilder<REv> {
     //
     // Note: This function is implemented manually without `async` sugar because the `Send`
     // inference seems to not work in all cases otherwise.
-    pub async fn fatal(self, file: &'static str, line: u32, msg: String)
+    pub(crate) async fn fatal(self, file: &'static str, line: u32, msg: String)
     where
         REv: From<ControlAnnouncement>,
     {
@@ -495,18 +510,6 @@ impl<REv> EffectBuilder<REv> {
         .await
     }
 
-    /// Retrieves block at `height` from the Linear Chain component.
-    pub(crate) async fn get_block_at_height_local<I>(self, height: u64) -> Option<Block>
-    where
-        REv: From<LinearChainRequest<I>>,
-    {
-        self.make_request(
-            |responder| LinearChainRequest::BlockAtHeightLocal(height, responder),
-            QueueKind::Regular,
-        )
-        .await
-    }
-
     /// Sends a network message.
     ///
     /// The message is queued in "fire-and-forget" fashion, there is no guarantee that the peer
@@ -529,7 +532,7 @@ impl<REv> EffectBuilder<REv> {
     /// Broadcasts a network message.
     ///
     /// Broadcasts a network message to all peers connected at the time the message is sent.
-    pub async fn broadcast_message<I, P>(self, payload: P)
+    pub(crate) async fn broadcast_message<I, P>(self, payload: P)
     where
         REv: From<NetworkRequest<I, P>>,
     {
@@ -549,7 +552,7 @@ impl<REv> EffectBuilder<REv> {
     /// excluding the indicated ones, and sends each a copy of the message.
     ///
     /// Returns the IDs of the chosen nodes.
-    pub async fn gossip_message<I, P>(
+    pub(crate) async fn gossip_message<I, P>(
         self,
         payload: P,
         count: usize,
@@ -573,7 +576,7 @@ impl<REv> EffectBuilder<REv> {
     }
 
     /// Gets connected network peers.
-    pub async fn network_peers<I>(self) -> BTreeMap<I, String>
+    pub(crate) async fn network_peers<I>(self) -> BTreeMap<I, String>
     where
         REv: From<NetworkInfoRequest<I>>,
         I: Send + 'static,
@@ -585,6 +588,32 @@ impl<REv> EffectBuilder<REv> {
         .await
     }
 
+    /// Gets the current network peers in a random order.
+    pub async fn get_peers_in_random_order<I>(self) -> Vec<I>
+    where
+        REv: From<NetworkInfoRequest<I>>,
+        I: Send + 'static,
+    {
+        self.make_request(
+            |responder| NetworkInfoRequest::GetPeersInRandomOrder { responder },
+            QueueKind::Api,
+        )
+        .await
+    }
+
+    /// Announces which deploys have expired.
+    pub(crate) async fn announce_expired_deploys(self, hashes: Vec<DeployHash>)
+    where
+        REv: From<BlockProposerAnnouncement>,
+    {
+        self.0
+            .schedule(
+                BlockProposerAnnouncement::DeploysExpired(hashes),
+                QueueKind::Regular,
+            )
+            .await;
+    }
+
     /// Announces that a network message has been received.
     pub(crate) async fn announce_message_received<I, P>(self, sender: I, payload: P)
     where
@@ -718,35 +747,6 @@ impl<REv> EffectBuilder<REv> {
             .await
     }
 
-    /// Announce that a block had been executed before.
-    pub(crate) async fn announce_block_already_executed(self, block: Block)
-    where
-        REv: From<ContractRuntimeAnnouncement>,
-    {
-        self.0
-            .schedule(
-                ContractRuntimeAnnouncement::block_already_executed(block),
-                QueueKind::Regular,
-            )
-            .await
-    }
-
-    /// Announce a committed Step success.
-    pub(crate) async fn announce_step_success(
-        self,
-        era_id: EraId,
-        execution_effect: ExecutionEffect,
-    ) where
-        REv: From<ContractRuntimeAnnouncement>,
-    {
-        self.0
-            .schedule(
-                ContractRuntimeAnnouncement::step_success(era_id, (&execution_effect).into()),
-                QueueKind::Regular,
-            )
-            .await
-    }
-
     /// Announce upgrade activation point read.
     pub(crate) async fn announce_upgrade_activation_point_read(self, next_upgrade: NextUpgrade)
     where
@@ -943,27 +943,16 @@ impl<REv> EffectBuilder<REv> {
             .await
     }
 
-    /// Requests the highest switch block.
-    // TODO - remove once used.
-    #[allow(unused)]
-    pub(crate) async fn get_highest_switch_block_from_storage(self) -> Option<Block>
-    where
-        REv: From<StorageRequest>,
-    {
-        self.make_request(
-            |responder| StorageRequest::GetHighestSwitchBlock { responder },
-            QueueKind::Regular,
-        )
-        .await
-    }
-
-    /// Read a trie by its hash key
-    pub(crate) async fn read_trie(self, trie_key: Blake2bHash) -> Option<Trie<Key, StoredValue>>
+    /// Get a trie by its hash key.
+    pub(crate) async fn get_trie(
+        self,
+        trie_key: Digest,
+    ) -> Result<Option<Trie<Key, StoredValue>>, engine_state::Error>
     where
         REv: From<ContractRuntimeRequest>,
     {
         self.make_request(
-            |responder| ContractRuntimeRequest::ReadTrie {
+            |responder| ContractRuntimeRequest::GetTrie {
                 trie_key,
                 responder,
             },
@@ -977,7 +966,7 @@ impl<REv> EffectBuilder<REv> {
     pub(crate) async fn put_trie_and_find_missing_descendant_trie_keys(
         self,
         trie: Box<Trie<Key, StoredValue>>,
-    ) -> Result<Vec<Blake2bHash>, engine_state::Error>
+    ) -> Result<Vec<Digest>, engine_state::Error>
     where
         REv: From<ContractRuntimeRequest>,
     {
@@ -1003,14 +992,14 @@ impl<REv> EffectBuilder<REv> {
     /// Gets the requested deploys from the deploy store.
     pub(crate) async fn get_deploys_from_storage(
         self,
-        deploy_hashes: Multiple<DeployHash>,
+        deploy_hashes: Vec<DeployHash>,
     ) -> Vec<Option<Deploy>>
     where
         REv: From<StorageRequest>,
     {
         self.make_request(
             |responder| StorageRequest::GetDeploys {
-                deploy_hashes: deploy_hashes.to_vec(),
+                deploy_hashes,
                 responder,
             },
             QueueKind::Regular,
@@ -1185,9 +1174,9 @@ impl<REv> EffectBuilder<REv> {
                 BlockProposerRequest::RequestBlockPayload(BlockPayloadRequest {
                     context,
                     next_finalized,
-                    responder,
                     accusations,
                     random_bit,
+                    responder,
                 })
             },
             QueueKind::Regular,
@@ -1195,14 +1184,54 @@ impl<REv> EffectBuilder<REv> {
         .await
     }
 
-    /// Passes a finalized proto-block to the block executor component to execute it.
-    pub(crate) async fn execute_block(self, finalized_block: FinalizedBlock)
+    /// Executes a finalized block.
+    pub(crate) async fn execute_finalized_block(
+        self,
+        protocol_version: ProtocolVersion,
+        execution_pre_state: ExecutionPreState,
+        finalized_block: FinalizedBlock,
+        deploys: Vec<Deploy>,
+        transfers: Vec<Deploy>,
+    ) -> Result<BlockAndExecutionEffects, BlockExecutionError>
     where
         REv: From<ContractRuntimeRequest>,
+    {
+        self.make_request(
+            |responder| ContractRuntimeRequest::ExecuteBlock {
+                protocol_version,
+                execution_pre_state,
+                finalized_block,
+                deploys,
+                transfers,
+                responder,
+            },
+            QueueKind::Regular,
+        )
+        .await
+    }
+
+    /// Enqueues a finalized proto-block execution.
+    ///
+    /// # Arguments
+    ///
+    /// * `finalized_block` - a finalized proto-block to add to the execution queue.
+    /// * `deploys` - a vector of deploys and transactions that match the hashes in the finalized
+    ///   block, in that order.
+    pub(crate) async fn enqueue_block_for_execution(
+        self,
+        finalized_block: FinalizedBlock,
+        deploys: Vec<Deploy>,
+        transfers: Vec<Deploy>,
+    ) where
+        REv: From<ContractRuntimeRequest>,
     {
         self.0
             .schedule(
-                ContractRuntimeRequest::ExecuteBlock(finalized_block),
+                ContractRuntimeRequest::EnqueueBlockForExecution {
+                    finalized_block,
+                    deploys,
+                    transfers,
+                },
                 QueueKind::Regular,
             )
             .await
@@ -1318,7 +1347,7 @@ impl<REv> EffectBuilder<REv> {
     pub(crate) async fn commit_genesis(
         self,
         chainspec: Arc<Chainspec>,
-    ) -> Result<GenesisResult, engine_state::Error>
+    ) -> Result<GenesisSuccess, engine_state::Error>
     where
         REv: From<ContractRuntimeRequest>,
     {
@@ -1336,7 +1365,7 @@ impl<REv> EffectBuilder<REv> {
     pub(crate) async fn upgrade_contract_runtime(
         self,
         upgrade_config: Box<UpgradeConfig>,
-    ) -> Result<UpgradeResult, engine_state::Error>
+    ) -> Result<UpgradeSuccess, engine_state::Error>
     where
         REv: From<ContractRuntimeRequest>,
     {
@@ -1371,38 +1400,6 @@ impl<REv> EffectBuilder<REv> {
         .await
     }
 
-    /// Loads potentially previously stored state from storage.
-    ///
-    /// Key must be a unique key across the the application, as all keys share a common namespace.
-    ///
-    /// If an error occurs during state loading or no data is found, returns `None`.
-    #[allow(unused)]
-    pub(crate) async fn load_state<T>(self, key: Cow<'static, [u8]>) -> Option<T>
-    where
-        REv: From<StateStoreRequest>,
-        T: DeserializeOwned,
-    {
-        // There is an ugly truth hidden in here: Due to object safety issues, we cannot ship the
-        // actual values around, but only the serialized bytes. For this reason this function
-        // retrieves raw bytes from storage and perform deserialization here.
-        //
-        // Errors are prominently logged but not treated further in any way.
-        self.make_request(
-            move |responder| StateStoreRequest::Load { key, responder },
-            QueueKind::Regular,
-        )
-        .await
-        .map(|data| bincode::deserialize(&data))
-        .transpose()
-        .unwrap_or_else(|err| {
-            let type_name = type_name::<T>();
-            panic!(
-                "could not deserialize state from storage type name {:?} err {:?}",
-                type_name, err
-            );
-        })
-    }
-
     /// Retrieves finalized deploys from blocks that were created more recently than the TTL.
     pub(crate) async fn get_finalized_deploys(
         self,
@@ -1469,29 +1466,75 @@ impl<REv> EffectBuilder<REv> {
         .await
     }
 
-    pub(crate) async fn is_verified_account(self, account_key: Key) -> Option<bool>
+    /// Retrieves an `Account` from global state if present.
+    pub(crate) async fn get_account_from_global_state(
+        self,
+        prestate_hash: Digest,
+        account_key: Key,
+    ) -> Option<Account>
     where
         REv: From<ContractRuntimeRequest>,
-        REv: From<StorageRequest>,
     {
-        if let Some(block) = self.get_highest_block_from_storage().await {
-            let state_hash = (*block.state_root_hash()).into();
-            let query_request = QueryRequest::new(state_hash, account_key, vec![]);
-            if let Ok(QueryResult::Success { value, .. }) =
-                self.query_global_state(query_request).await
-            {
-                if let StoredValue::Account(account) = *value {
-                    let purse_uref = account.main_purse();
-                    let balance_request = BalanceRequest::new(state_hash, purse_uref);
-                    if let Ok(balance_result) = self.get_balance(balance_request).await {
-                        if let Some(motes) = balance_result.motes() {
-                            return Some(motes >= &*MAX_PAYMENT);
-                        }
-                    }
+        let query_request = QueryRequest::new(prestate_hash, account_key, vec![]);
+        match self.query_global_state(query_request).await {
+            Ok(QueryResult::Success { value, .. }) => value.as_account().cloned(),
+            Ok(_) | Err(_) => None,
+        }
+    }
+
+    /// Retrieves the balance of a purse, returns `None` if no purse is present.
+    pub(crate) async fn check_purse_balance(
+        self,
+        prestate_hash: Digest,
+        main_purse: URef,
+    ) -> Option<U512>
+    where
+        REv: From<ContractRuntimeRequest>,
+    {
+        let balance_request = BalanceRequest::new(prestate_hash, main_purse);
+        match self.get_balance(balance_request).await {
+            Ok(balance_result) => {
+                if let Some(motes) = balance_result.motes() {
+                    return Some(*motes);
                 }
+                None
             }
+            Err(_) => None,
+        }
+    }
+
+    /// Retrieves an `Contract` from global state if present.
+    pub(crate) async fn get_contract_for_validation(
+        self,
+        prestate_hash: Digest,
+        query_key: Key,
+        path: Vec<String>,
+    ) -> Option<Contract>
+    where
+        REv: From<ContractRuntimeRequest>,
+    {
+        let query_request = QueryRequest::new(prestate_hash, query_key, path);
+        match self.query_global_state(query_request).await {
+            Ok(QueryResult::Success { value, .. }) => value.as_contract().cloned(),
+            Ok(_) | Err(_) => None,
+        }
+    }
+
+    /// Retrieves an `ContractPackage` from global state if present.
+    pub(crate) async fn get_contract_package_for_validation(
+        self,
+        prestate_hash: Digest,
+        query_key: Key,
+        path: Vec<String>,
+    ) -> Option<ContractPackage>
+    where
+        REv: From<ContractRuntimeRequest>,
+    {
+        let query_request = QueryRequest::new(prestate_hash, query_key, path);
+        match self.query_global_state(query_request).await {
+            Ok(QueryResult::Success { value, .. }) => value.as_contract_package().cloned(),
+            Ok(_) | Err(_) => None,
         }
-        None
     }
 
     /// Requests a query be executed on the Contract Runtime component.
@@ -1512,26 +1555,6 @@ impl<REv> EffectBuilder<REv> {
         .await
     }
 
-    /// Returns `ProtocolData` by `ProtocolVersion`.
-    ///
-    /// This operation is read only.
-    pub(crate) async fn get_protocol_data(
-        self,
-        protocol_version: ProtocolVersion,
-    ) -> Result<Option<Box<ProtocolData>>, engine_state::Error>
-    where
-        REv: From<ContractRuntimeRequest>,
-    {
-        self.make_request(
-            |responder| ContractRuntimeRequest::GetProtocolData {
-                protocol_version,
-                responder,
-            },
-            QueueKind::Regular,
-        )
-        .await
-    }
-
     /// Returns a map of validators weights for all eras as known from `root_hash`.
     ///
     /// This operation is read only.
@@ -1567,24 +1590,6 @@ impl<REv> EffectBuilder<REv> {
         .await
     }
 
-    /// Runs the end of era step using the system smart contract.
-    pub(crate) async fn run_step(
-        self,
-        step_request: StepRequest,
-    ) -> Result<StepResult, engine_state::Error>
-    where
-        REv: From<ContractRuntimeRequest>,
-    {
-        self.make_request(
-            |responder| ContractRuntimeRequest::Step {
-                step_request,
-                responder,
-            },
-            QueueKind::Regular,
-        )
-        .await
-    }
-
     /// Gets the correct era validators set for the given era.
     /// Takes emergency restarts into account based on the information from the chainspec loader.
     pub(crate) async fn get_era_validators(self, era_id: EraId) -> Option<BTreeMap<PublicKey, U512>>
@@ -1629,7 +1634,7 @@ impl<REv> EffectBuilder<REv> {
                 // above the key block for the era
                 .map_or(initial_state_root_hash, |hdr| *hdr.state_root_hash())
             };
-            let req = EraValidatorsRequest::new(root_hash.into(), protocol_version);
+            let req = EraValidatorsRequest::new(root_hash, protocol_version);
             self.get_era_validators_from_contract_runtime(req)
                 .await
                 .ok()
@@ -1649,7 +1654,7 @@ impl<REv> EffectBuilder<REv> {
                 // runtime
                 let highest_block = self.get_highest_block_from_storage().await?;
                 let req = EraValidatorsRequest::new(
-                    (*highest_block.header().state_root_hash()).into(),
+                    *highest_block.header().state_root_hash(),
                     protocol_version,
                 );
                 self.get_era_validators_from_contract_runtime(req)
@@ -1718,6 +1723,17 @@ impl<REv> EffectBuilder<REv> {
             .await
     }
 
+    /// Returns a list of validator status changes, by public key.
+    pub(crate) async fn get_consensus_validator_changes(
+        self,
+    ) -> BTreeMap<PublicKey, Vec<(EraId, ValidatorChange)>>
+    where
+        REv: From<ConsensusRequest>,
+    {
+        self.make_request(ConsensusRequest::ValidatorChanges, QueueKind::Regular)
+            .await
+    }
+
     /// Collects the key blocks for the eras identified by provided era IDs. Returns
     /// `Some(HashMap(era_id → block_header))` if all the blocks have been read correctly, and
     /// `None` if at least one was missing. The header for EraId `n` is from the key block for that
@@ -1756,6 +1772,6 @@ impl<REv> EffectBuilder<REv> {
 #[macro_export]
 macro_rules! fatal {
     ($effect_builder:expr, $($arg:tt)*) => {
-        $effect_builder.fatal(file!(), line!(), format_args!($($arg)*).to_string())
+        $effect_builder.fatal(file!(), line!(), format!($($arg)*))
     };
 }
