@@ -8,10 +8,10 @@
 //! While the [`main`](fn.main.html) function is the central entrypoint for the node application,
 //! its core event loop is found inside the [reactor](reactor/index.html).
 
-#![doc(html_root_url = "https://docs.rs/casper-node/1.3.4")]
+#![doc(html_root_url = "https://docs.rs/casper-node/1.4.1")]
 #![doc(
-    html_favicon_url = "https://raw.githubusercontent.com/casper-network/casper-node/master/images/CasperLabs_Logo_Favicon_RGB_50px.png",
-    html_logo_url = "https://raw.githubusercontent.com/casper-network/casper-node/master/images/CasperLabs_Logo_Symbol_RGB.png",
+    html_favicon_url = "https://raw.githubusercontent.com/CasperLabs/casper-node/master/images/CasperLabs_Logo_Favicon_RGB_50px.png",
+    html_logo_url = "https://raw.githubusercontent.com/CasperLabs/casper-node/master/images/CasperLabs_Logo_Symbol_RGB.png",
     test(attr(forbid(warnings)))
 )]
 #![warn(
@@ -20,23 +20,24 @@
     trivial_numeric_casts,
     unused_qualifications
 )]
-#![feature(test)]
 
-extern crate test;
-
-pub mod components;
+pub(crate) mod components;
 mod config_migration;
-pub mod crypto;
 mod data_migration;
-pub mod effect;
-pub mod logging;
-pub mod protocol;
-pub mod reactor;
+pub(crate) mod effect;
+pub(crate) mod logging;
+pub(crate) mod protocol;
+pub(crate) mod reactor;
 #[cfg(test)]
-pub mod testing;
-pub mod tls;
+pub(crate) mod testing;
+pub(crate) mod tls;
+pub(crate) mod utils;
+
+// Public API
+pub mod cli;
+pub mod crypto;
 pub mod types;
-pub mod utils;
+pub use components::rpc_server::rpcs;
 
 use std::sync::{
     atomic::{AtomicBool, AtomicUsize},
@@ -52,30 +53,31 @@ use signal_hook::{
     flag,
 };
 
-pub use components::{
-    block_proposer::Config as BlockProposerConfig,
-    consensus::Config as ConsensusConfig,
+pub(crate) use components::{
+    block_proposer::Config as BlockProposerConfig, consensus::Config as ConsensusConfig,
     contract_runtime::Config as ContractRuntimeConfig,
     deploy_acceptor::Config as DeployAcceptorConfig,
-    event_stream_server::Config as EventStreamServerConfig,
-    fetcher::Config as FetcherConfig,
-    gossiper::{Config as GossipConfig, Error as GossipError},
-    linear_chain_sync::Config as LinearChainSyncConfig,
-    rest_server::Config as RestServerConfig,
-    rpc_server::{rpcs, Config as RpcServerConfig},
-    small_network::{Config as SmallNetworkConfig, Error as SmallNetworkError},
-    storage::{Config as StorageConfig, Error as StorageError},
+    event_stream_server::Config as EventStreamServerConfig, fetcher::Config as FetcherConfig,
+    gossiper::Config as GossipConfig, linear_chain_sync::Config as LinearChainSyncConfig,
+    rest_server::Config as RestServerConfig, rpc_server::Config as RpcServerConfig,
+    small_network::Config as SmallNetworkConfig, storage::Config as StorageConfig,
 };
-pub use config_migration::{migrate_config, Error as ConfigMigrationError};
-pub use data_migration::{migrate_data, Error as DataMigrationError};
-pub use types::NodeRng;
-pub use utils::OS_PAGE_SIZE;
+pub(crate) use types::NodeRng;
 
 /// The maximum thread count which should be spawned by the tokio runtime.
 pub const MAX_THREAD_COUNT: usize = 512;
 
 fn version_string(color: bool) -> String {
-    let mut version = format!("{}-{}", env!("CARGO_PKG_VERSION"), env!("VERGEN_SHA_SHORT"));
+    let mut version = format!(
+        "{}-{}{}",
+        env!("CARGO_PKG_VERSION"),
+        env!("VERGEN_SHA_SHORT"),
+        if cfg!(feature = "casper-mainnet") {
+            "-casper-mainnet"
+        } else {
+            ""
+        }
+    );
 
     // Add a `@DEBUG` (or similar) tag to release string on non-release builds.
     if env!("NODE_BUILD_PROFILE") != "release" {
@@ -93,21 +95,21 @@ fn version_string(color: bool) -> String {
 
 /// Color version string for the compiled node. Filled in at build time, output allocated at
 /// runtime.
-pub static VERSION_STRING_COLOR: Lazy<String> = Lazy::new(|| version_string(true));
+pub(crate) static VERSION_STRING_COLOR: Lazy<String> = Lazy::new(|| version_string(true));
 
 /// Version string for the compiled node. Filled in at build time, output allocated at runtime.
-pub static VERSION_STRING: Lazy<String> = Lazy::new(|| version_string(false));
+pub(crate) static VERSION_STRING: Lazy<String> = Lazy::new(|| version_string(false));
 
 /// Global value that indicates the currently running reactor should exit if it is non-zero.
-pub static TERMINATION_REQUESTED: Lazy<Arc<AtomicUsize>> =
+pub(crate) static TERMINATION_REQUESTED: Lazy<Arc<AtomicUsize>> =
     Lazy::new(|| Arc::new(AtomicUsize::new(0)));
 
 /// Global flag that indicates the currently running reactor should dump its event queue.
-pub static QUEUE_DUMP_REQUESTED: Lazy<Arc<AtomicBool>> =
+pub(crate) static QUEUE_DUMP_REQUESTED: Lazy<Arc<AtomicBool>> =
     Lazy::new(|| Arc::new(AtomicBool::new(false)));
 
 /// Setup UNIX signal hooks for current application.
-pub fn setup_signal_hooks() {
+pub(crate) fn setup_signal_hooks() {
     for signal in TERM_SIGNALS {
         flag::register_usize(
             *signal,
@@ -121,12 +123,12 @@ pub fn setup_signal_hooks() {
 
 /// Constructs a new `NodeRng`.
 #[cfg(not(test))]
-pub fn new_rng() -> NodeRng {
+pub(crate) fn new_rng() -> NodeRng {
     NodeRng::from_entropy()
 }
 
 /// Constructs a new `NodeRng`.
 #[cfg(test)]
-pub fn new_rng() -> NodeRng {
+pub(crate) fn new_rng() -> NodeRng {
     NodeRng::new()
 }
