@@ -10,19 +10,19 @@ use serde::Serialize;
 use thiserror::Error;
 use tracing::info;
 
+use casper_execution_engine::core::engine_state;
+
 use crate::{
     components::{
         chainspec_loader::{self, ChainspecLoader},
-        contract_runtime::{self, ContractRuntime},
+        contract_runtime::{self, ContractRuntime, ContractRuntimeAnnouncement},
         gossiper,
         small_network::{GossipedAddress, SmallNetworkIdentity, SmallNetworkIdentityError},
         storage::{self, Storage},
         Component,
     },
     effect::{
-        announcements::{
-            ChainspecLoaderAnnouncement, ContractRuntimeAnnouncement, ControlAnnouncement,
-        },
+        announcements::{ChainspecLoaderAnnouncement, ControlAnnouncement},
         requests::{
             ConsensusRequest, ContractRuntimeRequest, LinearChainRequest, NetworkRequest,
             RestRequest, StateStoreRequest, StorageRequest,
@@ -39,7 +39,7 @@ use crate::{
 /// Top-level event for the reactor.
 #[derive(Debug, From, Serialize)]
 #[must_use]
-pub enum Event {
+pub(crate) enum Event {
     /// Chainspec handler event.
     #[from]
     Chainspec(chainspec_loader::Event),
@@ -50,8 +50,7 @@ pub enum Event {
     Storage(#[serde(skip_serializing)] storage::Event),
 
     /// Contract runtime event.
-    #[from]
-    ContractRuntime(#[serde(skip_serializing)] contract_runtime::Event),
+    ContractRuntime(#[serde(skip_serializing)] Box<ContractRuntimeRequest>),
 
     /// Request for state storage.
     #[from]
@@ -62,6 +61,12 @@ pub enum Event {
     ControlAnnouncement(ControlAnnouncement),
 }
 
+impl From<ContractRuntimeRequest> for Event {
+    fn from(contract_runtime_request: ContractRuntimeRequest) -> Self {
+        Event::ContractRuntime(Box::new(contract_runtime_request))
+    }
+}
+
 impl ReactorEvent for Event {
     fn as_control(&self) -> Option<&ControlAnnouncement> {
         if let Self::ControlAnnouncement(ref ctrl_ann) = self {
@@ -70,6 +75,16 @@ impl ReactorEvent for Event {
             None
         }
     }
+
+    fn description(&self) -> &'static str {
+        match self {
+            Event::Chainspec(_) => "Chainspec",
+            Event::Storage(_) => "Storage",
+            Event::ContractRuntime(_) => "ContractRuntime",
+            Event::StateStoreRequest(_) => "StateStoreRequest",
+            Event::ControlAnnouncement(_) => "ControlAnnouncement",
+        }
+    }
 }
 
 impl From<StorageRequest> for Event {
@@ -78,12 +93,6 @@ impl From<StorageRequest> for Event {
     }
 }
 
-impl From<ContractRuntimeRequest> for Event {
-    fn from(request: ContractRuntimeRequest) -> Self {
-        Event::ContractRuntime(contract_runtime::Event::Request(Box::new(request)))
-    }
-}
-
 impl From<NetworkRequest<NodeId, Message>> for Event {
     fn from(_request: NetworkRequest<NodeId, Message>) -> Self {
         unreachable!("no network traffic happens during initialization")
@@ -142,11 +151,7 @@ impl Display for Event {
 
 /// Error type returned by the initializer reactor.
 #[derive(Debug, Error)]
-pub enum Error {
-    /// `Config` error.
-    #[error("config error: {0}")]
-    ConfigError(String),
-
+pub(crate) enum Error {
     /// Metrics-related error
     #[error("prometheus (metrics) error: {0}")]
     Metrics(#[from] prometheus::Error),
@@ -165,12 +170,28 @@ pub enum Error {
 
     /// An error that occurred when creating a `SmallNetworkIdentity`.
     #[error(transparent)]
-    SmallNetworkIdentityError(#[from] SmallNetworkIdentityError),
+    SmallNetworkIdentity(#[from] SmallNetworkIdentityError),
+
+    /// An execution engine state error.
+    #[error(transparent)]
+    EngineState(#[from] engine_state::Error),
+
+    /// Trie key store is corrupted (missing trie keys).
+    #[error(
+        "Missing trie keys. Number of state roots: {state_root_count}, \
+         Number of missing trie keys: {missing_trie_key_count}"
+    )]
+    MissingTrieKeys {
+        /// The number of state roots in all of the block headers.
+        state_root_count: usize,
+        /// The number of trie keys we could not find.
+        missing_trie_key_count: usize,
+    },
 }
 
 /// Initializer node reactor.
 #[derive(DataSize, Debug)]
-pub struct Reactor {
+pub(crate) struct Reactor {
     pub(super) config: WithDir<participating::Config>,
     pub(super) chainspec_loader: ChainspecLoader,
     pub(super) storage: Storage,
@@ -193,14 +214,16 @@ impl Reactor {
             hard_reset_to_start_of_era,
             chainspec_loader.chainspec().protocol_config.version,
             crashed,
+            &chainspec_loader.chainspec().network_config.name,
         )?;
 
         let contract_runtime = ContractRuntime::new(
-            chainspec_loader.initial_state_root_hash(),
-            chainspec_loader.initial_block_header(),
             chainspec_loader.chainspec().protocol_config.version,
-            storage_config,
+            storage.root_path(),
             &config.value().contract_runtime,
+            chainspec_loader.chainspec().wasm_config,
+            chainspec_loader.chainspec().system_costs_config,
+            chainspec_loader.chainspec().core_config.max_associated_keys,
             registry,
         )?;
 
@@ -213,15 +236,13 @@ impl Reactor {
         // on restarts (online checks are an alternative).
         if crashed {
             info!("running trie-store integrity check, this may take a while");
-            if let Some(state_roots) = storage.get_state_root_hashes_for_trie_check() {
-                let missing_trie_keys = contract_runtime.trie_store_check(state_roots.clone());
-                if !missing_trie_keys.is_empty() {
-                    panic!(
-                        "Fatal error! Trie-Key store is not empty.\n {:?}\n \
-                        Wipe the DB to ensure operations.\n Present state_roots: {:?}",
-                        missing_trie_keys, state_roots
-                    )
-                }
+            let state_roots = storage.read_state_root_hashes_for_trie_check()?;
+            let missing_trie_keys = contract_runtime.trie_store_check(state_roots.clone())?;
+            if !missing_trie_keys.is_empty() {
+                return Err(Error::MissingTrieKeys {
+                    state_root_count: state_roots.len(),
+                    missing_trie_key_count: missing_trie_keys.len(),
+                });
             }
         }
 
@@ -243,7 +264,7 @@ impl Reactor {
 #[cfg(test)]
 impl Reactor {
     /// Inspect storage.
-    pub fn storage(&self) -> &Storage {
+    pub(crate) fn storage(&self) -> &Storage {
         &self.storage
     }
 }
@@ -284,9 +305,9 @@ impl reactor::Reactor for Reactor {
                 self.storage.handle_event(effect_builder, rng, event),
             ),
             Event::ContractRuntime(event) => reactor::wrap_effects(
-                Event::ContractRuntime,
+                Event::from,
                 self.contract_runtime
-                    .handle_event(effect_builder, rng, event),
+                    .handle_event(effect_builder, rng, *event),
             ),
             Event::StateStoreRequest(request) => {
                 self.dispatch_event(effect_builder, rng, Event::Storage(request.into()))
@@ -301,7 +322,7 @@ impl reactor::Reactor for Reactor {
 }
 
 #[cfg(test)]
-pub mod test {
+pub(crate) mod test {
     use super::*;
     use crate::{testing::network::NetworkedReactor, types::Chainspec};
     use std::sync::Arc;
