@@ -55,7 +55,7 @@ use rand::{
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use thiserror::Error;
 
-use crate::utils::{read_file, write_file};
+use crate::utils::read_file;
 
 // This is inside a private module so that the generated `BigArray` does not form part of this
 // crate's public API, and hence also doesn't appear in the rustdocs.
@@ -545,24 +545,6 @@ pub(crate) fn load_private_key<P: AsRef<Path>>(src: P) -> anyhow::Result<PKey<Pr
     PKey::private_key_from_pem(&pem).context("parsing private key")
 }
 
-/// Saves a certificate to a file.
-pub fn save_cert<P: AsRef<Path>>(cert: &X509Ref, dest: P) -> anyhow::Result<()> {
-    let pem = cert.to_pem().context("converting certificate to PEM")?;
-
-    write_file(dest, pem).with_context(|| "failed to write certificate")?;
-    Ok(())
-}
-
-/// Saves a private key to a file.
-pub fn save_private_key<P: AsRef<Path>>(key: &PKeyRef<Private>, dest: P) -> anyhow::Result<()> {
-    let pem = key
-        .private_key_to_pem_pkcs8()
-        .context("converting private key to PEM")?;
-
-    write_file(dest, pem).with_context(|| "failed to write private key")?;
-    Ok(())
-}
-
 /// Returns an OpenSSL compatible timestamp.
 fn now() -> i64 {
     // Note: We could do the timing dance a little better going straight to the UNIX time functions,
