@@ -4,6 +4,7 @@ pub(crate) mod appendable_block;
 mod block;
 pub mod chainspec;
 mod deploy;
+pub mod error;
 mod exit_code;
 mod item;
 pub mod json_compatibility;
@@ -19,17 +20,19 @@ use rand::{CryptoRng, RngCore};
 use rand_chacha::ChaCha20Rng;
 
 pub use block::{
-    json_compatibility::JsonBlock, Block, BlockBody, BlockHash, BlockHeader, BlockSignatures,
-    BlockValidationError, FinalitySignature,
+    json_compatibility::{JsonBlock, JsonBlockHeader},
+    Block, BlockBody, BlockHash, BlockHeader, BlockSignatures, FinalitySignature,
+    HashingAlgorithmVersion, MerkleBlockBody, MerkleBlockBodyPart, MerkleLinkedListNode,
 };
 pub(crate) use block::{BlockByHeight, BlockHeaderWithMetadata, BlockPayload, FinalizedBlock};
 pub(crate) use chainspec::ActivationPoint;
 pub use chainspec::Chainspec;
 pub use datasize::DataSize;
 pub use deploy::{
-    Approval, Deploy, DeployHash, DeployHeader, DeployMetadata, DeployOrTransferHash,
-    DeployValidationFailure, Error as DeployError, ExcessiveSizeError as ExcessiveSizeDeployError,
+    Approval, Deploy, DeployConfigurationFailure, DeployHash, DeployHeader, DeployMetadata,
+    DeployOrTransferHash, Error as DeployError, ExcessiveSizeError as ExcessiveSizeDeployError,
 };
+pub use error::BlockValidationError;
 pub use exit_code::ExitCode;
 pub use item::{Item, Tag};
 pub use node_config::NodeConfig;
