@@ -5,20 +5,17 @@
 use std::iter;
 use std::{
     array::TryFromSliceError,
+    cmp::Reverse,
     collections::BTreeMap,
     error::Error as StdError,
     fmt::{self, Debug, Display, Formatter},
-    hash::Hash,
-};
-
-use blake2::{
-    digest::{Update, VariableOutput},
-    VarBlake2b,
 };
 
 use datasize::DataSize;
+use derive_more::Into;
 use hex::FromHexError;
 use hex_fmt::HexList;
+use itertools::Itertools;
 use once_cell::sync::Lazy;
 #[cfg(test)]
 use rand::Rng;
@@ -26,6 +23,7 @@ use schemars::JsonSchema;
 use serde::{Deserialize, Serialize};
 use thiserror::Error;
 
+use casper_hashing::Digest;
 #[cfg(test)]
 use casper_types::system::auction::BLOCK_REWARD;
 use casper_types::{
@@ -34,21 +32,18 @@ use casper_types::{
 };
 
 use super::{Item, Tag, Timestamp};
-#[cfg(test)]
-use crate::crypto::generate_ed25519_keypair;
-#[cfg(test)]
-use crate::testing::TestRng;
 use crate::{
     components::consensus,
-    crypto::{
-        self,
-        hash::{self, Digest},
-        AsymmetricKeyExt,
-    },
+    crypto::{self, AsymmetricKeyExt},
     rpcs::docs::DocExample,
-    types::{Deploy, DeployHash, DeployOrTransferHash, JsonBlock},
+    types::{
+        error::{BlockCreationError, BlockValidationError},
+        Deploy, DeployHash, DeployOrTransferHash, JsonBlock, JsonBlockHeader,
+    },
     utils::DisplayIter,
 };
+#[cfg(test)]
+use crate::{crypto::generate_ed25519_keypair, testing::TestRng};
 
 static ERA_REPORT: Lazy<EraReport> = Lazy::new(|| {
     let secret_key_1 = SecretKey::ed25519_from_bytes([0; 32]).unwrap();
@@ -149,6 +144,7 @@ static BLOCK: Lazy<Block> = Lazy::new(|| {
         next_era_validator_weights,
         protocol_version,
     )
+    .expect("could not construct block")
 });
 static JSON_BLOCK: Lazy<JsonBlock> = Lazy::new(|| {
     let block = Block::doc_example().clone();
@@ -162,6 +158,10 @@ static JSON_BLOCK: Lazy<JsonBlock> = Lazy::new(|| {
 
     JsonBlock::new(block, Some(block_signature))
 });
+static JSON_BLOCK_HEADER: Lazy<JsonBlockHeader> = Lazy::new(|| {
+    let block_header = Block::doc_example().header().clone();
+    JsonBlockHeader::from(block_header)
+});
 
 /// Error returned from constructing or validating a `Block`.
 #[derive(Debug, Error)]
@@ -193,7 +193,9 @@ impl From<TryFromSliceError> for Error {
 /// From the view of the consensus protocol this is the "consensus value": The protocol deals with
 /// finalizing an order of `BlockPayload`s. Only after consensus has been reached, the block's
 /// deploys actually get executed, and the executed block gets signed.
-#[derive(Clone, DataSize, Debug, PartialOrd, Ord, PartialEq, Eq, Hash, Serialize, Deserialize)]
+#[derive(
+    Clone, DataSize, Debug, PartialOrd, Ord, PartialEq, Eq, Hash, Serialize, Deserialize, Default,
+)]
 pub struct BlockPayload {
     deploy_hashes: Vec<DeployHash>,
     transfer_hashes: Vec<DeployHash>,
@@ -373,20 +375,16 @@ impl FinalizedBlock {
         self.proposer.clone()
     }
 
-    /// Returns an iterator over all deploy and transfer hashes.
-    pub(crate) fn deploys_and_transfers_iter(
-        &self,
-    ) -> impl Iterator<Item = DeployOrTransferHash> + '_ {
-        self.deploy_hashes
-            .iter()
-            .copied()
-            .map(DeployOrTransferHash::Deploy)
-            .chain(
-                self.transfer_hashes
-                    .iter()
-                    .copied()
-                    .map(DeployOrTransferHash::Transfer),
-            )
+    /// Returns the WebAssembly-deploy hashes for the finalized block. These correspond to complex
+    /// smart contract operations that require a WebAssembly VM in the execution engine.
+    pub(crate) fn deploy_hashes(&self) -> &[DeployHash] {
+        &self.deploy_hashes
+    }
+
+    /// Returns the transfer hashes for the finalized block. These correspond to simple token
+    /// transfers that do not require a VM as part of their execution.
+    pub(crate) fn transfer_hashes(&self) -> &[DeployHash] {
+        &self.transfer_hashes
     }
 
     /// Generates a random instance using a `TestRng`.
@@ -408,9 +406,10 @@ impl FinalizedBlock {
         is_switch: bool,
     ) -> Self {
         let deploy_count = rng.gen_range(0..11);
-        let deploy_hashes = iter::repeat_with(|| DeployHash::new(Digest::random(rng)))
-            .take(deploy_count)
-            .collect();
+        let deploy_hashes =
+            iter::repeat_with(|| DeployHash::new(rng.gen::<[u8; Digest::LENGTH]>().into()))
+                .take(deploy_count)
+                .collect();
         let random_bit = rng.gen();
         // TODO - make Timestamp deterministic.
         let timestamp = Timestamp::now();
@@ -514,6 +513,7 @@ impl Display for FinalizedBlock {
     Deserialize,
     Debug,
     JsonSchema,
+    Into,
 )]
 #[serde(deny_unknown_fields)]
 pub struct BlockHash(Digest);
@@ -532,7 +532,7 @@ impl BlockHash {
     /// Creates a random block hash.
     #[cfg(test)]
     pub fn random(rng: &mut TestRng) -> Self {
-        let hash = Digest::random(rng);
+        let hash = rng.gen::<[u8; Digest::LENGTH]>().into();
         BlockHash(hash)
     }
 }
@@ -597,6 +597,31 @@ impl EraEnd {
     pub fn era_report(&self) -> &EraReport {
         &self.era_report
     }
+
+    pub fn hash(&self) -> Digest {
+        // Pattern match here leverages compiler to ensure every field is accounted for
+        let EraEnd {
+            next_era_validator_weights,
+            era_report,
+        } = self;
+        // Sort next era validator weights by descending weight. Assuming the top validators are
+        // online, a client can get away with just a few (plus a Merkle proof of the rest)
+        // to check finality signatures.
+        let descending_validator_weight_hashed_pairs: Vec<Digest> = next_era_validator_weights
+            .iter()
+            .sorted_by_key(|(_, weight)| Reverse(**weight))
+            .map(|(validator_id, weight)| {
+                let validator_hash =
+                    Digest::hash(validator_id.to_bytes().expect("Could not hash validator"));
+                let weight_hash = Digest::hash(weight.to_bytes().expect("Could not hash weight"));
+                Digest::hash_pair(&validator_hash, &weight_hash)
+            })
+            .collect();
+        let hashed_next_era_validator_weights =
+            Digest::hash_vec_merkle_tree(descending_validator_weight_hashed_pairs);
+        let hashed_era_report: Digest = era_report.hash();
+        Digest::hash_slice_rfold(&[hashed_next_era_validator_weights, hashed_era_report])
+    }
 }
 
 impl ToBytes for EraEnd {
@@ -652,6 +677,12 @@ pub struct BlockHeader {
 }
 
 impl BlockHeader {
+    /// The [`HashingAlgorithmVersion`] used for the header (as well as for its corresponding block
+    /// body).
+    pub fn hashing_algorithm_version(&self) -> HashingAlgorithmVersion {
+        HashingAlgorithmVersion::from_protocol_version(&self.protocol_version)
+    }
+
     /// The parent block's hash.
     pub fn parent_hash(&self) -> &BlockHash {
         &self.parent_hash
@@ -720,7 +751,8 @@ impl BlockHeader {
         self.era_end.is_some()
     }
 
-    /// The validators for the upcoming era and their respective weights.
+    /// The validators for the upcoming era and their respective weights (if this is a switch
+    /// block).
     pub fn next_era_validator_weights(&self) -> Option<&BTreeMap<PublicKey, U512>> {
         match &self.era_end {
             Some(era_end) => {
@@ -731,11 +763,75 @@ impl BlockHeader {
         }
     }
 
+    /// Takes the validators for the upcoming era and their respective weights (if this is a switch
+    /// block).
+    pub fn maybe_take_next_era_validator_weights(self) -> Option<BTreeMap<PublicKey, U512>> {
+        self.era_end
+            .map(|era_end| era_end.next_era_validator_weights)
+    }
+
     /// Hash of the block header.
     pub fn hash(&self) -> BlockHash {
+        match HashingAlgorithmVersion::from_protocol_version(&self.protocol_version) {
+            HashingAlgorithmVersion::V1 => self.hash_v1(),
+            HashingAlgorithmVersion::V2 => self.hash_v2(),
+        }
+    }
+
+    fn hash_v1(&self) -> BlockHash {
         let serialized_header = Self::serialize(self)
             .unwrap_or_else(|error| panic!("should serialize block header: {}", error));
-        BlockHash::new(hash::hash(&serialized_header))
+        BlockHash::new(Digest::hash(&serialized_header))
+    }
+
+    fn hash_v2(&self) -> BlockHash {
+        // Pattern match here leverages compiler to ensure every field is accounted for
+        let BlockHeader {
+            parent_hash,
+            era_id,
+            body_hash,
+            state_root_hash,
+            era_end,
+            height,
+            timestamp,
+            protocol_version,
+            random_bit,
+            accumulated_seed,
+        } = self;
+
+        let hashed_era_end = match era_end {
+            None => Digest::SENTINEL_NONE,
+            Some(era_end) => era_end.hash(),
+        };
+
+        let hashed_era_id = Digest::hash(era_id.to_bytes().expect("Could not serialize era_id"));
+        let hashed_height = Digest::hash(height.to_bytes().expect("Could not serialize height"));
+        let hashed_timestamp =
+            Digest::hash(timestamp.to_bytes().expect("Could not serialize timestamp"));
+        let hashed_protocol_version = Digest::hash(
+            protocol_version
+                .to_bytes()
+                .expect("Could not serialize protocol version"),
+        );
+        let hashed_random_bit = Digest::hash(
+            random_bit
+                .to_bytes()
+                .expect("Could not serialize protocol version"),
+        );
+
+        Digest::hash_slice_rfold(&[
+            hashed_protocol_version,
+            parent_hash.0,
+            hashed_era_end,
+            *body_hash,
+            hashed_era_id,
+            *state_root_hash,
+            hashed_height,
+            hashed_timestamp,
+            hashed_random_bit,
+            *accumulated_seed,
+        ])
+        .into()
     }
 
     /// Returns true if block is Genesis' child.
@@ -828,7 +924,7 @@ impl FromBytes for BlockHeader {
     }
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq)]
 pub struct BlockHeaderWithMetadata {
     pub block_header: BlockHeader,
     pub block_signatures: BlockSignatures,
@@ -840,6 +936,139 @@ impl Display for BlockHeaderWithMetadata {
     }
 }
 
+/// A node in a Merkle linked-list used for hashing data structures.
+#[derive(Debug, Clone)]
+pub struct MerkleLinkedListNode<T> {
+    value: T,
+    merkle_proof_of_rest: Digest,
+}
+
+impl<T> MerkleLinkedListNode<T> {
+    /// Constructs a new [`MerkleLinkedListNode`].
+    pub fn new(value: T, merkle_proof_of_rest: Digest) -> MerkleLinkedListNode<T> {
+        MerkleLinkedListNode {
+            value,
+            merkle_proof_of_rest,
+        }
+    }
+
+    /// Gets the hash of the rest of the linked list.
+    pub fn merkle_proof_of_rest(&self) -> &Digest {
+        &self.merkle_proof_of_rest
+    }
+
+    /// Converts a [`MerkleLinkedListNode`] into its underlying value.
+    pub fn take_value(self) -> T {
+        self.value
+    }
+}
+
+/// A fragment of a Merkle-treeified [`BlockBody`].
+///
+/// Has the following hash structure:
+///
+/// ```text
+/// merkle_linked_list_node_hash
+///     |          \
+/// value_hash      merkle_linked_list_node::merkle_proof_of_rest
+///     |
+/// merkle_linked_list_node::value
+/// ```
+#[derive(Debug, Clone)]
+pub struct MerkleBlockBodyPart<'a, T> {
+    value_hash: Digest,
+    merkle_linked_list_node: MerkleLinkedListNode<&'a T>,
+    merkle_linked_list_node_hash: Digest,
+}
+
+impl<'a, T> MerkleBlockBodyPart<'a, T> {
+    fn new(value: &T, value_hash: Digest, merkle_proof_of_rest: Digest) -> MerkleBlockBodyPart<T> {
+        MerkleBlockBodyPart {
+            value_hash,
+            merkle_linked_list_node: MerkleLinkedListNode {
+                value,
+                merkle_proof_of_rest,
+            },
+            merkle_linked_list_node_hash: Digest::hash_pair(&value_hash, &merkle_proof_of_rest),
+        }
+    }
+
+    /// The value of the block body part
+    pub fn value(&self) -> &'a T {
+        self.merkle_linked_list_node.value
+    }
+
+    /// The hash of the value and the rest of the linked list as a slice
+    pub fn value_and_rest_hashes_pair(&self) -> (Digest, Digest) {
+        (
+            self.value_hash,
+            self.merkle_linked_list_node.merkle_proof_of_rest,
+        )
+    }
+
+    /// The hash of the value of the block body part
+    pub fn value_hash(&self) -> &Digest {
+        &self.value_hash
+    }
+
+    /// The hash of the linked list node
+    pub fn merkle_linked_list_node_hash(&self) -> &Digest {
+        &self.merkle_linked_list_node_hash
+    }
+}
+
+/// A [`BlockBody`] that has been hashed so its parts may be stored as a Merkle linked-list.
+///
+/// ```text
+///                   body_hash (root)
+///                   /      \__________________
+/// hash(deploy_hashes)      /                  \_____________
+///                   hash(transfer_hashes)     /             \
+///                                         hash(proposer)   SENTINEL
+/// ```
+#[derive(Debug, Clone)]
+pub struct MerkleBlockBody<'a> {
+    /// Merklized list of the deploy hashes from the block body.
+    pub deploy_hashes: MerkleBlockBodyPart<'a, Vec<DeployHash>>,
+    /// Merklized list of the transfer hashes from the block body.
+    pub transfer_hashes: MerkleBlockBodyPart<'a, Vec<DeployHash>>,
+    /// Merklized [`BlockBody::proposer`].
+    pub proposer: MerkleBlockBodyPart<'a, PublicKey>,
+}
+
+#[cfg(test)]
+impl<'a> MerkleBlockBody<'a> {
+    /// Takes the hashes and Merkle proofs for a [`MerkleBlockBody`].
+    /// The `Digest` triplets contain the node hash, and contained within that node: the value hash
+    /// and the Merkle proof of the rest of the tree.
+    pub(crate) fn take_hashes_and_proofs(
+        self,
+    ) -> [(Digest, Digest, Digest); BlockBody::PARTS_COUNT] {
+        let MerkleBlockBody {
+            deploy_hashes,
+            transfer_hashes,
+            proposer,
+        } = self;
+        [
+            (
+                deploy_hashes.merkle_linked_list_node_hash,
+                deploy_hashes.value_hash,
+                deploy_hashes.merkle_linked_list_node.merkle_proof_of_rest,
+            ),
+            (
+                transfer_hashes.merkle_linked_list_node_hash,
+                transfer_hashes.value_hash,
+                transfer_hashes.merkle_linked_list_node.merkle_proof_of_rest,
+            ),
+            (
+                proposer.merkle_linked_list_node_hash,
+                proposer.value_hash,
+                proposer.merkle_linked_list_node.merkle_proof_of_rest,
+            ),
+        ]
+    }
+}
+
 /// The body portion of a block.
 #[derive(Clone, DataSize, Ord, PartialOrd, Eq, PartialEq, Hash, Serialize, Deserialize, Debug)]
 pub struct BlockBody {
@@ -849,6 +1078,9 @@ pub struct BlockBody {
 }
 
 impl BlockBody {
+    /// The number of parts of a block body.
+    pub const PARTS_COUNT: usize = 3;
+
     /// Creates a new body from deploy and transfer hashes.
     pub(crate) fn new(
         proposer: PublicKey,
@@ -877,12 +1109,61 @@ impl BlockBody {
         &self.transfer_hashes
     }
 
-    /// Computes the body hash
-    pub(crate) fn hash(&self) -> Digest {
+    /// Computes the body hash by hashing the serialized bytes.
+    fn hash_v1(&self) -> Digest {
         let serialized_body = self
             .to_bytes()
             .unwrap_or_else(|error| panic!("should serialize block body: {}", error));
-        hash::hash(&serialized_body)
+        Digest::hash(&serialized_body)
+    }
+
+    /// Constructs the block body hashes for the block body.
+    pub fn merklize(&self) -> MerkleBlockBody {
+        // Pattern match here leverages compiler to ensure every field is accounted for
+        let BlockBody {
+            deploy_hashes,
+            transfer_hashes,
+            proposer,
+        } = self;
+
+        let proposer = MerkleBlockBodyPart::new(
+            proposer,
+            Digest::hash(&proposer.to_bytes().expect("Could not serialize proposer")),
+            Digest::SENTINEL_RFOLD,
+        );
+
+        let transfer_hashes = MerkleBlockBodyPart::new(
+            transfer_hashes,
+            Digest::hash_vec_merkle_tree(
+                transfer_hashes.iter().cloned().map(Digest::from).collect(),
+            ),
+            proposer.merkle_linked_list_node_hash,
+        );
+
+        let deploy_hashes = MerkleBlockBodyPart::new(
+            deploy_hashes,
+            Digest::hash_vec_merkle_tree(deploy_hashes.iter().cloned().map(Digest::from).collect()),
+            transfer_hashes.merkle_linked_list_node_hash,
+        );
+
+        MerkleBlockBody {
+            deploy_hashes,
+            transfer_hashes,
+            proposer,
+        }
+    }
+
+    /// Computes the block body hash.
+    fn hash_v2(&self) -> Digest {
+        self.merklize().deploy_hashes.merkle_linked_list_node_hash
+    }
+
+    /// Computes the block body hash, using the indicated hashing algorithm version.
+    pub fn hash(&self, version: HashingAlgorithmVersion) -> Digest {
+        match version {
+            HashingAlgorithmVersion::V1 => self.hash_v1(),
+            HashingAlgorithmVersion::V2 => self.hash_v2(),
+        }
     }
 }
 
@@ -923,43 +1204,8 @@ impl FromBytes for BlockBody {
     }
 }
 
-/// An error that can arise when validating a block's cryptographic integrity using its hashes
-#[derive(Debug)]
-pub enum BlockValidationError {
-    /// Problem serializing some of a block's data into bytes
-    SerializationError(bytesrepr::Error),
-
-    /// The body hash in the header is not the same as the hash of the body of the block
-    UnexpectedBodyHash {
-        /// The block body hash specified in the header that is apparently incorrect
-        expected_by_block_header: Digest,
-        /// The actual hash of the block's body
-        actual: Digest,
-    },
-
-    /// The block's hash is not the same as the header's hash
-    UnexpectedBlockHash {
-        /// The hash specified by the block
-        expected_by_block: BlockHash,
-        /// The actual hash of the block
-        actual: BlockHash,
-    },
-}
-
-impl Display for BlockValidationError {
-    fn fmt(&self, formatter: &mut Formatter) -> fmt::Result {
-        write!(formatter, "{:?}", self)
-    }
-}
-
-impl From<bytesrepr::Error> for BlockValidationError {
-    fn from(err: bytesrepr::Error) -> Self {
-        BlockValidationError::SerializationError(err)
-    }
-}
-
 /// A storage representation of finality signatures with the associated block hash.
-#[derive(Debug, Serialize, Deserialize, Clone, DataSize, Eq, PartialEq)]
+#[derive(Clone, Debug, PartialOrd, Ord, Hash, Serialize, Deserialize, DataSize, Eq, PartialEq)]
 pub struct BlockSignatures {
     /// The block hash for a given block.
     pub(crate) block_hash: BlockHash,
@@ -1026,7 +1272,42 @@ pub struct Block {
     body: BlockBody,
 }
 
+/// The hashing algorithm used for the header and the body of a block
+#[derive(PartialEq, Eq, Clone, Copy, Debug)]
+pub enum HashingAlgorithmVersion {
+    /// Version 1
+    V1,
+    /// Version 2
+    V2,
+}
+
+impl HashingAlgorithmVersion {
+    #[cfg(feature = "casper-mainnet")]
+    pub(crate) const HASH_V2_PROTOCOL_VERSION: ProtocolVersion =
+        // TODO: restore that to the actual switchover version when fast sync is merged
+        ProtocolVersion::from_parts(9001, 0, 0);
+
+    #[cfg(not(feature = "casper-mainnet"))]
+    pub(crate) const HASH_V2_PROTOCOL_VERSION: ProtocolVersion =
+        ProtocolVersion::from_parts(0, 0, 0);
+
+    fn from_protocol_version(protocol_version: &ProtocolVersion) -> Self {
+        if *protocol_version < Self::HASH_V2_PROTOCOL_VERSION {
+            HashingAlgorithmVersion::V1
+        } else {
+            HashingAlgorithmVersion::V2
+        }
+    }
+}
+
 impl Block {
+    fn hash_block_body(protocol_version: &ProtocolVersion, block_body: &BlockBody) -> Digest {
+        match HashingAlgorithmVersion::from_protocol_version(protocol_version) {
+            HashingAlgorithmVersion::V1 => block_body.hash_v1(),
+            HashingAlgorithmVersion::V2 => block_body.hash_v2(),
+        }
+    }
+
     pub(crate) fn new(
         parent_hash: BlockHash,
         parent_seed: Digest,
@@ -1034,33 +1315,36 @@ impl Block {
         finalized_block: FinalizedBlock,
         next_era_validator_weights: Option<BTreeMap<PublicKey, U512>>,
         protocol_version: ProtocolVersion,
-    ) -> Self {
+    ) -> Result<Self, BlockCreationError> {
         let body = BlockBody::new(
             finalized_block.proposer.clone(),
             finalized_block.deploy_hashes,
             finalized_block.transfer_hashes,
         );
-        let body_hash = body.hash();
 
-        let era_end = finalized_block
-            .era_report
-            .map(|era_report| EraEnd::new(era_report, next_era_validator_weights.unwrap()));
+        let body_hash = Self::hash_block_body(&protocol_version, &body);
 
-        let mut accumulated_seed = [0; Digest::LENGTH];
+        let era_end = match (finalized_block.era_report, next_era_validator_weights) {
+            (None, None) => None,
+            (Some(era_report), Some(next_era_validator_weights)) => {
+                Some(EraEnd::new(era_report, next_era_validator_weights))
+            }
+            (maybe_era_report, maybe_next_era_validator_weights) => {
+                return Err(BlockCreationError::CouldNotCreateEraEnd {
+                    maybe_era_report,
+                    maybe_next_era_validator_weights,
+                })
+            }
+        };
 
-        let mut hasher = VarBlake2b::new(Digest::LENGTH).expect("should create hasher");
-        hasher.update(parent_seed);
-        hasher.update([finalized_block.random_bit as u8]);
-        hasher.finalize_variable(|slice| {
-            accumulated_seed.copy_from_slice(slice);
-        });
+        let accumulated_seed = Digest::hash_pair(parent_seed, [finalized_block.random_bit as u8]);
 
         let header = BlockHeader {
             parent_hash,
             state_root_hash,
             body_hash,
             random_bit: finalized_block.random_bit,
-            accumulated_seed: accumulated_seed.into(),
+            accumulated_seed,
             era_end,
             timestamp: finalized_block.timestamp,
             era_id: finalized_block.era_id,
@@ -1068,12 +1352,21 @@ impl Block {
             protocol_version,
         };
 
-        Self::new_from_header_and_body(header, body)
+        Ok(Block {
+            hash: header.hash(),
+            header,
+            body,
+        })
     }
 
-    pub(crate) fn new_from_header_and_body(header: BlockHeader, body: BlockBody) -> Self {
+    pub(crate) fn new_from_header_and_body(
+        header: BlockHeader,
+        body: BlockBody,
+    ) -> Result<Self, BlockValidationError> {
         let hash = header.hash();
-        Block { hash, header, body }
+        let block = Block { hash, header, body };
+        block.verify()?;
+        Ok(block)
     }
 
     pub(crate) fn header(&self) -> &BlockHeader {
@@ -1135,20 +1428,23 @@ impl Block {
 
     /// Check the integrity of a block by hashing its body and header
     pub fn verify(&self) -> Result<(), BlockValidationError> {
-        let actual_body_hash = self.body.hash();
-        if self.header.body_hash != actual_body_hash {
-            return Err(BlockValidationError::UnexpectedBodyHash {
-                expected_by_block_header: self.header.body_hash,
-                actual: actual_body_hash,
-            });
-        }
-        let actual_header_hash = self.header.hash();
-        if self.hash != actual_header_hash {
+        let actual_block_header_hash = self.header().hash();
+        if *self.hash() != actual_block_header_hash {
             return Err(BlockValidationError::UnexpectedBlockHash {
-                expected_by_block: self.hash,
-                actual: actual_header_hash,
+                block: Box::new(self.to_owned()),
+                actual_block_header_hash,
             });
         }
+
+        let actual_block_body_hash =
+            Self::hash_block_body(&self.header.protocol_version, &self.body);
+        if self.header.body_hash != actual_block_body_hash {
+            return Err(BlockValidationError::UnexpectedBodyHash {
+                block: Box::new(self.to_owned()),
+                actual_block_body_hash,
+            });
+        }
+
         Ok(())
     }
 
@@ -1163,7 +1459,7 @@ impl Block {
     /// Generates a random instance using a `TestRng`.
     #[cfg(test)]
     pub fn random(rng: &mut TestRng) -> Self {
-        let era = rng.gen_range(0..5);
+        let era = rng.gen_range(1..6);
         let height = era * 10 + rng.gen_range(0..10);
         let is_switch = rng.gen_bool(0.1);
 
@@ -1185,10 +1481,10 @@ impl Block {
         protocol_version: ProtocolVersion,
         is_switch: bool,
     ) -> Self {
-        let parent_hash = BlockHash::new(Digest::random(rng));
-        let state_root_hash = Digest::random(rng);
+        let parent_hash = BlockHash::new(rng.gen::<[u8; Digest::LENGTH]>().into());
+        let state_root_hash = rng.gen::<[u8; Digest::LENGTH]>().into();
         let finalized_block = FinalizedBlock::random_with_specifics(rng, era_id, height, is_switch);
-        let parent_seed = Digest::random(rng);
+        let parent_seed = rng.gen::<[u8; Digest::LENGTH]>().into();
         let next_era_validator_weights = finalized_block
             .era_report
             .as_ref()
@@ -1202,6 +1498,7 @@ impl Block {
             next_era_validator_weights,
             protocol_version,
         )
+        .expect("Could not create random block with specifics")
     }
 }
 
@@ -1380,7 +1677,7 @@ pub(crate) mod json_compatibility {
 
     #[derive(Serialize, Deserialize, Debug, JsonSchema, Clone, PartialEq, Eq, DataSize)]
     #[serde(deny_unknown_fields)]
-    struct JsonEraEnd {
+    pub struct JsonEraEnd {
         era_report: JsonEraReport,
         next_era_validator_weights: Vec<ValidatorWeight>,
     }
@@ -1417,19 +1714,30 @@ pub(crate) mod json_compatibility {
         }
     }
 
+    /// JSON representation of a block header.
     #[derive(Serialize, Deserialize, Debug, JsonSchema, Clone, PartialEq, Eq, DataSize)]
     #[serde(deny_unknown_fields)]
-    struct JsonBlockHeader {
-        parent_hash: BlockHash,
-        state_root_hash: Digest,
-        body_hash: Digest,
-        random_bit: bool,
-        accumulated_seed: Digest,
-        era_end: Option<JsonEraEnd>,
-        timestamp: Timestamp,
-        era_id: EraId,
-        height: u64,
-        protocol_version: ProtocolVersion,
+    pub struct JsonBlockHeader {
+        /// The parent hash.
+        pub parent_hash: BlockHash,
+        /// The state root hash.
+        pub state_root_hash: Digest,
+        /// The body hash.
+        pub body_hash: Digest,
+        /// Randomness bit.
+        pub random_bit: bool,
+        /// Accumulated seed.
+        pub accumulated_seed: Digest,
+        /// The era end.
+        pub era_end: Option<JsonEraEnd>,
+        /// The block timestamp.
+        pub timestamp: Timestamp,
+        /// The block era id.
+        pub era_id: EraId,
+        /// The block height.
+        pub height: u64,
+        /// The protocol version.
+        pub protocol_version: ProtocolVersion,
     }
 
     impl From<BlockHeader> for JsonBlockHeader {
@@ -1466,6 +1774,12 @@ pub(crate) mod json_compatibility {
         }
     }
 
+    impl DocExample for JsonBlockHeader {
+        fn doc_example() -> &'static Self {
+            &*JSON_BLOCK_HEADER
+        }
+    }
+
     /// A JSON-friendly representation of `Body`
     #[derive(Serialize, Deserialize, Debug, JsonSchema, Clone, PartialEq, Eq, DataSize)]
     #[serde(deny_unknown_fields)]
@@ -1499,10 +1813,14 @@ pub(crate) mod json_compatibility {
     #[derive(Serialize, Deserialize, Debug, JsonSchema, Clone, PartialEq, Eq, DataSize)]
     #[serde(deny_unknown_fields)]
     pub struct JsonBlock {
-        hash: BlockHash,
-        header: JsonBlockHeader,
-        body: JsonBlockBody,
-        proofs: Vec<JsonProof>,
+        /// `BlockHash`
+        pub hash: BlockHash,
+        /// JSON-friendly block header.
+        pub header: JsonBlockHeader,
+        /// JSON-friendly block body.
+        pub body: JsonBlockBody,
+        /// JSON-friendly list of proofs for this block.
+        pub proofs: Vec<JsonProof>,
     }
 
     impl JsonBlock {
@@ -1607,7 +1925,7 @@ impl FinalitySignature {
         secret_key: &SecretKey,
         public_key: PublicKey,
     ) -> Self {
-        let mut bytes = block_hash.inner().to_vec();
+        let mut bytes = block_hash.inner().into_vec();
         bytes.extend_from_slice(&era_id.to_le_bytes());
         let signature = crypto::sign(bytes, secret_key, &public_key);
         FinalitySignature {
@@ -1621,7 +1939,7 @@ impl FinalitySignature {
     /// Verifies whether the signature is correct.
     pub fn verify(&self) -> crypto::Result<()> {
         // NOTE: This needs to be in sync with the `new` constructor.
-        let mut bytes = self.block_hash.inner().to_vec();
+        let mut bytes = self.block_hash.inner().into_vec();
         bytes.extend_from_slice(&self.era_id.to_le_bytes());
         crypto::verify(bytes, &self.signature, &self.public_key)
     }
@@ -1645,11 +1963,13 @@ impl Display for FinalitySignature {
 
 #[cfg(test)]
 mod tests {
+    use std::rc::Rc;
+
     use casper_types::bytesrepr;
 
-    use super::*;
     use crate::testing::TestRng;
-    use std::rc::Rc;
+
+    use super::*;
 
     #[test]
     fn json_block_roundtrip() {
@@ -1723,17 +2043,20 @@ mod tests {
         let mut rng = TestRng::from_seed([2u8; 16]);
         let mut block = Block::random(&mut rng);
 
-        let bogus_block_hash = hash::hash(&[0xde, 0xad, 0xbe, 0xef]);
-        block.header.body_hash = bogus_block_hash;
-
-        let actual_body_hash = block.body.hash();
+        let bogus_block_body_hash = Digest::hash(&[0xde, 0xad, 0xbe, 0xef]);
+        block.header.body_hash = bogus_block_body_hash;
+        block.hash = block.header.hash();
+        let bogus_block_hash = block.hash;
 
         // No Eq trait for BlockValidationError, so pattern match
         match block.verify() {
             Err(BlockValidationError::UnexpectedBodyHash {
-                expected_by_block_header,
-                actual,
-            }) if expected_by_block_header == bogus_block_hash && actual == actual_body_hash => {}
+                block,
+                actual_block_body_hash,
+            }) if block.hash == bogus_block_hash
+                && block.header.body_hash == bogus_block_body_hash
+                && block.body.hash(block.header.hashing_algorithm_version())
+                    == actual_block_body_hash => {}
             unexpected => panic!("Bad check response: {:?}", unexpected),
         }
     }
@@ -1743,17 +2066,16 @@ mod tests {
         let mut rng = TestRng::from_seed([3u8; 16]);
         let mut block = Block::random(&mut rng);
 
-        let bogus_block_hash: BlockHash = hash::hash(&[0xde, 0xad, 0xbe, 0xef]).into();
+        let bogus_block_hash: BlockHash = Digest::hash(&[0xde, 0xad, 0xbe, 0xef]).into();
         block.hash = bogus_block_hash;
 
-        let actual_block_hash = block.header.hash();
-
         // No Eq trait for BlockValidationError, so pattern match
         match block.verify() {
             Err(BlockValidationError::UnexpectedBlockHash {
-                expected_by_block,
-                actual,
-            }) if expected_by_block == bogus_block_hash && actual == actual_block_hash => {}
+                block,
+                actual_block_header_hash,
+            }) if block.hash == bogus_block_hash
+                && block.header.hash() == actual_block_header_hash => {}
             unexpected => panic!("Bad check response: {:?}", unexpected),
         }
     }
@@ -1779,4 +2101,39 @@ mod tests {
         // Test should fail b/c `signature` is over `era_id=1` and here we're using `era_id=2`.
         assert!(fs_manufactured.verify().is_err());
     }
+
+    #[test]
+    fn block_body_merkle_proof_should_be_correct() {
+        let mut rng = TestRng::new();
+        let era_id = rng.gen_range(0..10).into();
+        let height = rng.gen_range(0..100);
+        let is_switch = rng.gen();
+        let block = Block::random_with_specifics(
+            &mut rng,
+            era_id,
+            height,
+            HashingAlgorithmVersion::HASH_V2_PROTOCOL_VERSION,
+            is_switch,
+        );
+
+        let merkle_block_body = block.body().merklize();
+
+        let hashes = [
+            *merkle_block_body.deploy_hashes.value_hash(),
+            *merkle_block_body.transfer_hashes.value_hash(),
+            *merkle_block_body.proposer.value_hash(),
+        ];
+
+        assert_eq!(
+            block.header().body_hash(),
+            merkle_block_body
+                .deploy_hashes
+                .merkle_linked_list_node_hash()
+        );
+
+        assert_eq!(
+            *block.header().body_hash(),
+            Digest::hash_slice_rfold(&hashes[..])
+        );
+    }
 }
